﻿using Microsoft.Xna.Framework;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Gbjam5MksGame
{
    public class AnimatedMapItem
    {
        protected MyGame myGame;

        public MapTileItem[] MapItems;

        public bool AutoFrameAdvance;
        public bool StopOnPlayComplete;
        public bool IsPlaying;
        public int FrameIndex;
        public int NosFrames;

        public int frameHoldCounter;
        public int FrameHold;

        public int LayerIndex;

        public AnimatedMapItem()
        {
        }

        public virtual void PrepareItem(MyGame game)
        {
            myGame = game;

            MapItems = null;
            AutoFrameAdvance = true;
            StopOnPlayComplete = true;
            IsPlaying = false;
            FrameIndex = 0;
            NosFrames = 0;

            frameHoldCounter = 0;
            FrameHold = 15;

            AddAllSpritesToComponents();
        }

        public virtual void UnprepareItem()
        {
            RemoveAllSpritesFromComponents();
        }

        public void Show(bool flag)
        {
            if (MapItems == null)
            {
                return;
            }

            for (int index = 0; index < MapItems.Length; index++)
            {
                if (MapItems[index].Sprite != null)
                {
                    if (flag)
                    {
                        if (index == FrameIndex)
                        {
                            MapItems[index].Sprite.Visible = true;
                        }
                        else
                        {
                            MapItems[index].Sprite.Visible = false;
                        }
                    }
                    else
                    {
                        MapItems[index].Sprite.Visible = false;
                    }
                }
            }
        }

        public void SetPosition(Vector2 newPosition)
        {
            if (MapItems == null)
            {
                return;
            }

            for (int index = 0; index < MapItems.Length; index++)
            {
                if (MapItems[index].Sprite != null)
                {
                    MapItems[index].Position = newPosition;
                }
            }
        }

        public Vector2 GetPosition()
        {
            if (MapItems == null)
            {
                return Vector2.Zero;
            }

            if (MapItems.Length > 0)
            {
                return MapItems[0].Position;
            }

            return Vector2.Zero;
        }

        public void SetHeight(float newHeight)
        {
            if (MapItems == null)
            {
                return;
            }

            for (int index = 0; index < MapItems.Length; index++)
            {
                if (MapItems[index] != null)
                {
                    MapItems[index].Height = newHeight;

                    if (MapItems[index].Sprite != null)
                    {
                        MapItems[index].Sprite.DrawOrder = (int)FindTileLayerForHeight(MapItems[index].Height); 
                    }
                }
            }
        }

        public float GetHeight()
        {
            if (MapItems == null)
            {
                return 0.0f;
            }

            if (MapItems.Length > 0)
            {
                return MapItems[0].Height;
            }

            return 0.0f;
        }

        public DisplayLayer GetDrawOrder()
        {
            if (MapItems == null)
            {
                return DisplayLayer.MapLayer0Tiles;
            }

            if (MapItems.Length > 0)
            {
                if (MapItems[0].Sprite != null)
                {
                    return (DisplayLayer)MapItems[0].Sprite.DrawOrder;
                }
            }

            return DisplayLayer.MapLayer0Tiles;
        }

        public void UpdateBasedOnMapCenterPosition(Vector2 mapCenterPosition, Vector2 screenAnchorPosition)
        {
            if (MapItems == null)
            {
                return;
            }
        
            for (int index = 0; index < MapItems.Length; index++)
            {
                if (MapItems[index].Sprite != null)
                {
                    MapItems[index].Sprite.TargetPosition.X = MapItems[index].Position.X - mapCenterPosition.X - (MapItems[index].Sprite.TargetSize.X / 2);
                    MapItems[index].Sprite.TargetPosition.Y = MapItems[index].Position.Y - mapCenterPosition.Y - (MapItems[index].Sprite.TargetSize.Y / 2) - MapItems[index].Height;

                    MapItems[index].Sprite.TargetPosition.X += screenAnchorPosition.X;
                    MapItems[index].Sprite.TargetPosition.Y += screenAnchorPosition.Y;
                }
            }

            if (IsPlaying && AutoFrameAdvance)
            {
                frameHoldCounter++;
                if (frameHoldCounter >= FrameHold)
                {
                    frameHoldCounter = 0;

                    FrameIndex++;
                    if (FrameIndex >= NosFrames)
                    {
                        if (StopOnPlayComplete)
                        {
                            IsPlaying = false;
                        }
                        else
                        {
                            FrameIndex = 0;
                        }
                    }
                }
            }
        }

        protected virtual void AddAllSpritesToComponents()
        {
            if (MapItems == null)
            {
                return;
            }

            foreach (MapTileItem mapItem in MapItems)
            {
                if (mapItem != null && mapItem.Sprite != null)
                {
                    myGame.Components.Add(mapItem.Sprite);
                }
            }
        }

        protected virtual void RemoveAllSpritesFromComponents()
        {
            if (MapItems == null)
            {
                return;
            }

            foreach (MapTileItem mapItem in MapItems)
            {
                if (mapItem != null && mapItem.Sprite != null)
                {
                    myGame.Components.Remove(mapItem.Sprite);
                }
            }
        }

        DisplayLayer FindTileLayerForHeight(float height)
        {
            DisplayLayer result = DisplayLayer.MapLayer0Tiles;

            int index = (int)(height / 7.0f);
            if (index >= 0 && index <= 4)
            {
                switch (index)
                {
                    case 0:
                        result = DisplayLayer.MapLayer0Tiles;
                        break;
                    case 1:
                        result = DisplayLayer.MapLayer1Tiles;
                        break;
                    case 2:
                        result = DisplayLayer.MapLayer2Tiles;
                        break;
                    case 3:
                        result = DisplayLayer.MapLayer3Tiles;
                        break;
                    case 4:
                        result = DisplayLayer.MapLayer4Tiles;
                        break;
                    default:
                        break;
                }
            }

            return result;
        }
    }
}
