﻿using Microsoft.Xna.Framework;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Gbjam5MksGame
{
    public class EnemyMissile : AnimatedMapItem    
    {
        public int LifetimeFrames;
        public Vector2 DirectionUnitVector; // x, y in map
        public float HeightUnitValue;  // is height change unit vector value
        public float Speed = 0.0f;

        public int SensorRate = 15;
        int currentSensorCount = 0;

        public EnemyMissile()
        {
        }

        public void Move(Vector2 targetPosition, float targetHeight)
        {
            if (MapItems.Length <= 0)
            {
                return;
            }

            if (currentSensorCount == 0)
            {
                Vector2 diff = targetPosition - MapItems[0].Position;
                diff.Normalize();

                DirectionUnitVector = diff;

                double atan2Value = Math.Atan2(diff.Y, diff.X);
                double angleDegrees = (atan2Value * 180.0) / Math.PI;

                while (angleDegrees < 0.0f)
                {
                    angleDegrees += 360.0f;
                }
                while (angleDegrees >= 360.0f)
                {
                    angleDegrees -= 360.0f;
                }

                //
                if (angleDegrees > 337.5 || angleDegrees <= 22.5)
                {
                    FrameIndex = 5;
                }
                else if (angleDegrees > 22.5 && angleDegrees <= 67.5)
                {
                    FrameIndex = 6;
                }
                else if (angleDegrees > 67.5 && angleDegrees <= 112.5)
                {
                    FrameIndex = 7;
                }
                else if (angleDegrees > 112.5 && angleDegrees <= 157.5)
                {
                    FrameIndex = 0;
                }
                else if (angleDegrees > 157.5 && angleDegrees <= 202.5)
                {
                    FrameIndex = 1;
                }
                else if (angleDegrees > 202.5 && angleDegrees <= 247.5)
                {
                    FrameIndex = 2;
                }
                else if (angleDegrees > 247.5 && angleDegrees <= 292.5)
                {
                    FrameIndex = 3;
                }
                else if (angleDegrees > 292.5 && angleDegrees <= 337.5)
                {
                    FrameIndex = 4;
                }

                currentSensorCount = SensorRate;

                System.Diagnostics.Debug.WriteLine("angle: " + angleDegrees.ToString() + ", FrameIndex: " + FrameIndex.ToString());

            }

            for (int index = 0; index < NosFrames; index++)
            {
                MapItems[index].SetPosition(MapItems[index].GetPosition() + (DirectionUnitVector * Speed));

                if (MapItems[index].GetHeight() < targetHeight)
                {
                    MapItems[index].SetHeight(MapItems[index].GetHeight() + (0.25f * Speed));
                }
                else if (MapItems[index].GetHeight() > targetHeight)
                {
                    MapItems[index].SetHeight(MapItems[index].GetHeight() - (0.25f * Speed));
                }

            }

            Show(true);

            currentSensorCount--;
        }

        public override void PrepareItem(MyGame game)
        {
            base.PrepareItem(game);

            AutoFrameAdvance = false;
            StopOnPlayComplete = false;
            IsPlaying = false;
            FrameIndex = 0;
            NosFrames = 8;
            FrameHold = 1;
            LifetimeFrames = 480;

            DirectionUnitVector = new Vector2(0.0f, 0.0f);
            HeightUnitValue = 0.0f; 
            Speed = 0.0f;

            MapItems = new MapTileItem[NosFrames];
            for (int index = 0; index < NosFrames; index++)
            {
                MapItems[index] = new MapTileItem();

                MapItems[index].Type = MapItemType.EnemyMissile;

                MapItems[index].Position = new Vector2(0.0f, 0.0f);
                MapItems[index].Height = 0.0f;
                MapItems[index].PixelSize = new int[2] { 16, 16 };
                MapItems[index].State = MapItemState.None;
                MapItems[index].HitPoints = 1;

                MapItems[index].Sprite = new SpriteGameVisual(myGame);
                MapItems[index].Sprite.DrawOrder = (int)DisplayLayer.MapLayer4Sprites;
                MapItems[index].Sprite.Enabled = false;
                MapItems[index].Sprite.UpdateOrder = (int)UpdateLayer.Last;
                MapItems[index].Sprite.SourceImage = myGame.TilesTexture;
                MapItems[index].Sprite.SourceRectangle = new Rectangle((index * 16), 80, 16, 16);
                MapItems[index].Sprite.TargetPosition = new Vector2(0.0f, 0.0f);
                MapItems[index].Sprite.TargetSize = new Vector2(16.0f, 16.0f);
                MapItems[index].Sprite.Visible = false;
            }

            AddAllSpritesToComponents();
        }

        protected override void AddAllSpritesToComponents()
        {
            if (MapItems == null)
            {
                return;
            }

            foreach (MapTileItem mapItem in MapItems)
            {
                if (mapItem != null && mapItem.Sprite != null)
                {
                    myGame.Components.Add(mapItem.Sprite);
                }
            }
        }

        protected override void  RemoveAllSpritesFromComponents()
        {
            if (MapItems == null)
            {
                return;
            }

            foreach (MapTileItem mapItem in MapItems)
            {
                if (mapItem != null && mapItem.Sprite != null)
                {
                    myGame.Components.Remove(mapItem.Sprite);
                }
            }
        }
    }
}
