﻿using Microsoft.Xna.Framework;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Gbjam5MksGame
{
    public class EnemyShot : AnimatedMapItem    
    {
        public int LifetimeFrames;
        public Vector2 DirectionUnitVector; // x, y in map
        public float HeightUnitValue;  // is height change unit vector value
        public float Speed = 0.0f;

        public EnemyShot()
        {
        }

        public void Move()
        {
            for (int index = 0; index < NosFrames; index++)
            {
                MapItems[index].Position.X += DirectionUnitVector.X * Speed;
                MapItems[index].Position.Y += DirectionUnitVector.Y * Speed;
                MapItems[index].Height += HeightUnitValue * Speed;
            }
        }

        public override void PrepareItem(MyGame game)
        {
            base.PrepareItem(game);

            AutoFrameAdvance = true;
            StopOnPlayComplete = false;
            IsPlaying = false;
            FrameIndex = 0;
            NosFrames = 3;
            FrameHold = 1;
            LifetimeFrames = 120;

            DirectionUnitVector = new Vector2(0.0f, 0.0f);
            HeightUnitValue = 0.0f; 
            Speed = 0.0f;

            MapItems = new MapTileItem[NosFrames];
            for (int index = 0; index < NosFrames; index++)
            {
                MapItems[index] = new MapTileItem();

                MapItems[index].Type = MapItemType.EnemyShot;

                MapItems[index].Position = new Vector2(0.0f, 0.0f);
                MapItems[index].Height = 0.0f;
                MapItems[index].PixelSize = new int[2] { 4, 4 };
                MapItems[index].State = MapItemState.None;
                MapItems[index].HitPoints = 1;

                MapItems[index].Sprite = new SpriteGameVisual(myGame);
                MapItems[index].Sprite.DrawOrder = (int)DisplayLayer.MapLayer4Sprites;
                MapItems[index].Sprite.Enabled = false;
                MapItems[index].Sprite.UpdateOrder = (int)UpdateLayer.Last;
                MapItems[index].Sprite.SourceImage = myGame.TilesTexture;
                MapItems[index].Sprite.SourceRectangle = new Rectangle(34 + (index * 4), 320, 4, 4);
                MapItems[index].Sprite.TargetPosition = new Vector2(0.0f, 0.0f);
                MapItems[index].Sprite.TargetSize = new Vector2(4.0f, 4.0f);
                MapItems[index].Sprite.Visible = true;
            }

            AddAllSpritesToComponents();
        }

        protected override void AddAllSpritesToComponents()
        {
            if (MapItems == null)
            {
                return;
            }

            foreach (MapTileItem mapItem in MapItems)
            {
                if (mapItem != null && mapItem.Sprite != null)
                {
                    myGame.Components.Add(mapItem.Sprite);
                }
            }
        }

        protected override void  RemoveAllSpritesFromComponents()
        {
            if (MapItems == null)
            {
                return;
            }

            foreach (MapTileItem mapItem in MapItems)
            {
                if (mapItem != null && mapItem.Sprite != null)
                {
                    myGame.Components.Remove(mapItem.Sprite);
                }
            }
        }
    }
}
