﻿using Microsoft.Xna.Framework;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Gbjam5MksGame
{
    public enum GroundEnemyType { None, Turret1, Turret2, Turret3, Turret4, Bunker1, RocketLauncher1 }

    public class GroundEnemy
    {
        LevelManager myLevelManager = null;

        public GroundEnemyType Type;

        public int LastShotCounter;
        public int MaxShotCounter;

        public int mapTileItemIndex;

        List<EnemyShot> shots = new List<EnemyShot>();
        List<EnemyMissile> missiles = new List<EnemyMissile>();
        List<SmallExplosion> smallExplosions = new List<SmallExplosion>();

        public float ViewRange;

        public GroundEnemy()
        {
            Type = GroundEnemyType.None;

            LastShotCounter = 0;
            MaxShotCounter = 5 * 60;

            mapTileItemIndex = -1;

            ViewRange = 0.0f;
        }

        public void Think()
        {
            UpdateShotBasedOnMapCenterPosition();
            UpdateMissileBasedOnMapCenterPosition();
            UpdateSmallExplosionBasedOnMapCenterPosition();

            Vector2 enemyPosition = myLevelManager.MapTileItems[mapTileItemIndex].Position + new Vector2(8.0f, 12.0f);
            float enemyHeight = myLevelManager.MapTileItems[mapTileItemIndex].Height;

            Vector2 distanceVector2D = myLevelManager.PlayerOffsetMapPosition - enemyPosition;
            float distanceVectorSightRange = distanceVector2D.Length();

            if (LastShotCounter == 0)
            {
                if (distanceVectorSightRange <= ViewRange)
                {
                    LastShotCounter = MaxShotCounter;

                    if (Type == GroundEnemyType.Turret1 && enemyPosition.X >= myLevelManager.PlayerOffsetMapPosition.X && enemyPosition.Y <= myLevelManager.PlayerOffsetMapPosition.Y)
                    {
                        Vector3 playerCurrentDirection = new Vector3(myLevelManager.PlayerOffsetMapPosition.X - enemyPosition.X, myLevelManager.PlayerOffsetMapPosition.Y - enemyPosition.Y, myLevelManager.PlayerMapHeight - enemyHeight);
                        Vector3 playerCurrentDirectionNormalized = playerCurrentDirection;
                        playerCurrentDirectionNormalized.Normalize();

                        Vector2 shot2dDirection = new Vector2(playerCurrentDirectionNormalized.X, playerCurrentDirectionNormalized.Y);

                        float shot2dHeight = playerCurrentDirectionNormalized.Z;

                        SpawnShot(enemyPosition, enemyHeight, shot2dDirection, shot2dHeight, myLevelManager.MyManager.GetLevelProgressSpeedFactor());

                        if (myLevelManager.MyGame.SfxOn)
                        {
                            myLevelManager.MyGame.EnemyFireShotSound.Play();
                        }
                    }
                    else if (Type == GroundEnemyType.Turret2 && enemyPosition.X >= myLevelManager.PlayerOffsetMapPosition.X && enemyPosition.Y >= myLevelManager.PlayerOffsetMapPosition.Y)
                    {
                        Vector3 playerCurrentDirection = new Vector3(myLevelManager.PlayerOffsetMapPosition.X - enemyPosition.X, myLevelManager.PlayerOffsetMapPosition.Y - enemyPosition.Y, myLevelManager.PlayerMapHeight - enemyHeight);
                        Vector3 playerCurrentDirectionNormalized = playerCurrentDirection;
                        playerCurrentDirectionNormalized.Normalize();

                        Vector2 shot2dDirection = new Vector2(playerCurrentDirectionNormalized.X, playerCurrentDirectionNormalized.Y);

                        float shot2dHeight = playerCurrentDirectionNormalized.Z;

                        SpawnShot(enemyPosition, enemyHeight, shot2dDirection, shot2dHeight, myLevelManager.MyManager.GetLevelProgressSpeedFactor());

                        if (myLevelManager.MyGame.SfxOn)
                        {
                            myLevelManager.MyGame.EnemyFireShotSound.Play();
                        }
                    }
                    else if (Type == GroundEnemyType.Turret3 && enemyPosition.X <= myLevelManager.PlayerOffsetMapPosition.X && enemyPosition.Y <= myLevelManager.PlayerOffsetMapPosition.Y)
                    {
                        Vector3 playerCurrentDirection = new Vector3(myLevelManager.PlayerOffsetMapPosition.X - enemyPosition.X, myLevelManager.PlayerOffsetMapPosition.Y - enemyPosition.Y, myLevelManager.PlayerMapHeight - enemyHeight);
                        Vector3 playerCurrentDirectionNormalized = playerCurrentDirection;
                        playerCurrentDirectionNormalized.Normalize();

                        Vector2 shot2dDirection = new Vector2(playerCurrentDirectionNormalized.X, playerCurrentDirectionNormalized.Y);

                        float shot2dHeight = playerCurrentDirectionNormalized.Z;

                        SpawnShot(enemyPosition, enemyHeight, shot2dDirection, shot2dHeight, myLevelManager.MyManager.GetLevelProgressSpeedFactor());

                        if (myLevelManager.MyGame.SfxOn)
                        {
                            myLevelManager.MyGame.EnemyFireShotSound.Play();
                        }
                    }
                    else if (Type == GroundEnemyType.Turret4 && enemyPosition.X <= myLevelManager.PlayerOffsetMapPosition.X && enemyPosition.Y <= myLevelManager.PlayerOffsetMapPosition.Y)
                    {
                        Vector3 playerCurrentDirection = new Vector3(myLevelManager.PlayerOffsetMapPosition.X - enemyPosition.X, myLevelManager.PlayerOffsetMapPosition.Y - enemyPosition.Y, myLevelManager.PlayerMapHeight - enemyHeight);
                        Vector3 playerCurrentDirectionNormalized = playerCurrentDirection;
                        playerCurrentDirectionNormalized.Normalize();

                        Vector2 shot2dDirection = new Vector2(playerCurrentDirectionNormalized.X, playerCurrentDirectionNormalized.Y);

                        float shot2dHeight = playerCurrentDirectionNormalized.Z;

                        SpawnShot(enemyPosition, enemyHeight, shot2dDirection, shot2dHeight, myLevelManager.MyManager.GetLevelProgressSpeedFactor());

                        if (myLevelManager.MyGame.SfxOn)
                        {
                            myLevelManager.MyGame.EnemyFireShotSound.Play();
                        }
                    }
                    else if (Type == GroundEnemyType.Bunker1 && enemyPosition.X >= myLevelManager.PlayerOffsetMapPosition.X)
                    {
                        Vector3 playerCurrentDirectionUpper = new Vector3(myLevelManager.PlayerOffsetMapPosition.X - enemyPosition.X - 16.0f, myLevelManager.PlayerOffsetMapPosition.Y - enemyPosition.Y - 8.0f, myLevelManager.PlayerMapHeight - enemyHeight);
                        Vector3 playerCurrentDirectionUpperNormalized = playerCurrentDirectionUpper;
                        playerCurrentDirectionUpperNormalized.Normalize();

                        Vector3 playerCurrentDirectionMiddle = new Vector3(myLevelManager.PlayerOffsetMapPosition.X - enemyPosition.X, myLevelManager.PlayerOffsetMapPosition.Y - enemyPosition.Y, myLevelManager.PlayerMapHeight - enemyHeight);
                        Vector3 playerCurrentDirectionMiddleNormalized = playerCurrentDirectionMiddle;
                        playerCurrentDirectionMiddleNormalized.Normalize();

                        Vector3 playerCurrentDirectionLower = new Vector3(myLevelManager.PlayerOffsetMapPosition.X - enemyPosition.X + 16.0f, myLevelManager.PlayerOffsetMapPosition.Y - enemyPosition.Y + 8.0f, myLevelManager.PlayerMapHeight - enemyHeight);
                        Vector3 playerCurrentDirectionLowerNormalized = playerCurrentDirectionLower;
                        playerCurrentDirectionLowerNormalized.Normalize();

                        Vector2 shot2dDirectionUpper = new Vector2(playerCurrentDirectionUpperNormalized.X, playerCurrentDirectionUpperNormalized.Y);
                        Vector2 shot2dDirectionMiddle = new Vector2(playerCurrentDirectionMiddleNormalized.X, playerCurrentDirectionMiddleNormalized.Y);
                        Vector2 shot2dDirectionLower = new Vector2(playerCurrentDirectionLowerNormalized.X, playerCurrentDirectionLowerNormalized.Y);

                        float shot2dHeight = playerCurrentDirectionMiddleNormalized.Z;

                        SpawnShot(enemyPosition, enemyHeight, shot2dDirectionUpper, shot2dHeight, myLevelManager.MyManager.GetLevelProgressSpeedFactor());
                        SpawnShot(enemyPosition, enemyHeight, shot2dDirectionMiddle, shot2dHeight, myLevelManager.MyManager.GetLevelProgressSpeedFactor());
                        SpawnShot(enemyPosition, enemyHeight, shot2dDirectionLower, shot2dHeight, myLevelManager.MyManager.GetLevelProgressSpeedFactor());

                        if (myLevelManager.MyGame.SfxOn)
                        {
                            myLevelManager.MyGame.EnemyFireShotSound.Play();
                        }
                    }
                    else if (Type == GroundEnemyType.RocketLauncher1 && enemyPosition.X >= myLevelManager.PlayerOffsetMapPosition.X)
                    {
                        Vector3 playerCurrentDirection = new Vector3(myLevelManager.PlayerOffsetMapPosition.X - enemyPosition.X, myLevelManager.PlayerOffsetMapPosition.Y - enemyPosition.Y, myLevelManager.PlayerMapHeight - enemyHeight);
                        Vector3 playerCurrentDirectionNormalized = playerCurrentDirection;
                        playerCurrentDirectionNormalized.Normalize();

                        Vector2 shot2dDirection = new Vector2(playerCurrentDirectionNormalized.X, playerCurrentDirectionNormalized.Y);

                        float shot2dHeight = playerCurrentDirectionNormalized.Z;

                        SpawnMissile(enemyPosition, enemyHeight, shot2dDirection, shot2dHeight, 0.25f * myLevelManager.MyManager.GetLevelProgressSpeedFactor());

                        if (myLevelManager.MyGame.SfxOn)
                        {
                            myLevelManager.MyGame.EnemyFireMissileSound.Play();
                        }
                    }
                }
            }
            else
            {
                LastShotCounter--;
            }
        }

        void SpawnShot(Vector2 mapPosition, float mapHeight, Vector2 newDirection, float newHeight, float newSpeed)
        {
            EnemyShot newShot = new EnemyShot();
            newShot.PrepareItem(myLevelManager.MyGame);
            newShot.SetPosition(mapPosition);
            newShot.SetHeight(mapHeight);
            newShot.IsPlaying = true;

            newShot.DirectionUnitVector = newDirection;
            newShot.HeightUnitValue = newHeight;

            newShot.Speed = newSpeed;

            shots.Add(newShot);
        }

        void UpdateShotBasedOnMapCenterPosition()
        {
            if (shots.Count <= 0)
            {
                return;
            }

            for (int index = shots.Count - 1; index >= 0; index--)
            {
                EnemyShot shot = shots[index];
                if (shot.IsPlaying)
                {
                    shot.Move();

                    shot.UpdateBasedOnMapCenterPosition(myLevelManager.PlayerMapPosition, myLevelManager.MapScreenCenterAnchorPosition);

                    shot.Show(true);

                    shot.LifetimeFrames--;
                    if (shot.LifetimeFrames <= 0)
                    {
                        shot.Show(false);

                        shot.IsPlaying = false;
                        shot.UnprepareItem();

                        shots.RemoveAt(index);

                        SpawnSmallExplosion(shot.GetPosition(), shot.GetHeight());
                    }
                    else
                    {
                        // check for collision with player
                        Vector2 positionDifference = myLevelManager.PlayerOffsetMapPosition - shot.GetPosition();
                        float positionDifferenceLength = positionDifference.Length();

                        float heightDifferenceLength = Math.Abs(myLevelManager.PlayerMapHeight - shot.GetHeight());

                        if (positionDifferenceLength <= 8.0f && heightDifferenceLength <= 4.0f)
                        {
                            shot.Show(false);

                            shot.IsPlaying = false;
                            shot.UnprepareItem();

                            shots.RemoveAt(index);

                            myLevelManager.DestroyPlayer();
                        }
                    }
                }
            }
        }

        void SpawnMissile(Vector2 mapPosition, float mapHeight, Vector2 newDirection, float newHeight, float newSpeed)
        {
            EnemyMissile newMissile = new EnemyMissile();
            newMissile.PrepareItem(myLevelManager.MyGame);
            newMissile.SetPosition(mapPosition);
            newMissile.SetHeight(mapHeight);
            newMissile.IsPlaying = true;

            newMissile.DirectionUnitVector = newDirection;
            newMissile.HeightUnitValue = newHeight;

            newMissile.Speed = newSpeed;

            missiles.Add(newMissile);
        }

        void UpdateMissileBasedOnMapCenterPosition()
        {
            if (missiles.Count <= 0)
            {
                return;
            }

            for (int index = missiles.Count - 1; index >= 0; index--)
            {
                EnemyMissile missile = missiles[index];
                if (missile.IsPlaying)
                {
                    missile.Move(myLevelManager.PlayerOffsetMapPosition, myLevelManager.PlayerMapHeight);

                    missile.UpdateBasedOnMapCenterPosition(myLevelManager.PlayerMapPosition, myLevelManager.MapScreenCenterAnchorPosition);

                    missile.Show(true);

                    missile.LifetimeFrames--;
                    if (missile.LifetimeFrames <= 0)
                    {
                        missile.Show(false);

                        missile.IsPlaying = false;
                        missile.UnprepareItem();

                        missiles.RemoveAt(index);

                        SpawnSmallExplosion(missile.GetPosition(), missile.GetHeight());
                    }
                    else
                    {
                        // check for collision with player
                        Vector2 positionDifference = myLevelManager.PlayerOffsetMapPosition - missile.GetPosition();
                        float positionDifferenceLength = positionDifference.Length();

                        float heightDifferenceLength = Math.Abs(myLevelManager.PlayerMapHeight - missile.GetHeight());

                        if (positionDifferenceLength <= 8.0f && heightDifferenceLength <= 4.0f)
                        {
                            missile.Show(false);

                            missile.IsPlaying = false;
                            missile.UnprepareItem();

                            missiles.RemoveAt(index);

                            myLevelManager.DestroyPlayer();
                        }
                    }
                }
            }
        }

        void SpawnSmallExplosion(Vector2 mapPosition, float mapHeight)
        {
            SmallExplosion newExplosion = new SmallExplosion();
            newExplosion.PrepareItem(myLevelManager.MyGame);
            newExplosion.SetPosition(mapPosition);
            newExplosion.SetHeight(mapHeight);
            newExplosion.IsPlaying = true;

            smallExplosions.Add(newExplosion);
        }

        void UpdateSmallExplosionBasedOnMapCenterPosition()
        {
            if (smallExplosions.Count <= 0)
            {
                return;
            }

            for (int index = smallExplosions.Count - 1; index >= 0; index--)
            {
                SmallExplosion explosion = smallExplosions[index];
                if (explosion.IsPlaying)
                {
                    explosion.UpdateBasedOnMapCenterPosition(myLevelManager.PlayerMapPosition, myLevelManager.MapScreenCenterAnchorPosition);

                    explosion.Show(true);
                }
                else
                {
                    explosion.Show(false);
                    explosion.UnprepareItem();

                    smallExplosions.RemoveAt(index);
                }
            }
        }

        public void Prepare(LevelManager newLevelManager)
        {
            myLevelManager = newLevelManager;
        }

        public void Unprepare()
        {
            foreach (EnemyShot shot in shots)
            {
                shot.Show(false);
                shot.UnprepareItem();
            }
            shots.Clear();

            foreach (EnemyMissile missile in missiles)
            {
                missile.Show(false);
                missile.UnprepareItem();
            }
            missiles.Clear();

            foreach (SmallExplosion explosion in smallExplosions)
            {
                explosion.Show(false);
                explosion.UnprepareItem();
            }
            smallExplosions.Clear();
        }
    }
}
