﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;

namespace Gbjam5MksGame
{
    public class LevelManager
    {
        public MyGame MyGame;
        public MapManagerGameComponent MyManager;

        XmlDocument doc;

        public int MapWidth = 0;
        public int MapHeight = 0;

        bool isValidLevelLoaded = false;
        public bool IsValidLevelLoaded
        {
            get
            {
                return isValidLevelLoaded;
            }
        }

        public Vector2 MapScreenCenterAnchorPosition = new Vector2(48.0f, 123.0f);
        public Vector2 PlayerMapPosition = new Vector2(8.0f, 8.0f);
        public float PlayerMapHeight; // = 0.0f;
        public Vector2 PlayerOffsetMapPosition = new Vector2(0.0f, 0.0f);
        public float PlayerShotSpeed = 4.0f;

        int[][] mapBinaryData = new int[5][];

        int maxShotFrameInterval = 30;
        int shotFrameInterval = 0;

        int craterTileNr = 7;

        // All the stuff in the (map) world
        public MapTileItem[] MapTileItems = null;
        List<PlayerShot> playerShots = new List<PlayerShot>();
        List<LargeExplosion> largeExplosions = new List<LargeExplosion>();
        List<MediumExplosion> mediumExplosions = new List<MediumExplosion>();
        List<SmallExplosion> smallExplosions = new List<SmallExplosion>();

        // The intelligent stuff in the game world
        List<GroundEnemy> groundEnemyList = new List<GroundEnemy>();

        public LevelManager()
        {
            isValidLevelLoaded = false;
        }

        public void SubPrepare(MyGame game, MapManagerGameComponent manager, int levelNr)
        {
            MyGame = game;
            MyManager = manager;

            string levelFilename = string.Format("..\\..\\..\\..\\Levels\\level{0}.tmx", levelNr);
            LoadXmlFile(levelFilename);
        }

        public void SubUnprepare()
        {
            foreach (MapTileItem tileItem in MapTileItems)
            {
                if (tileItem != null)
                {
                    if (tileItem.Sprite != null)
                    {
                        MyGame.Components.Remove(tileItem.Sprite);
                    }
                }
            }
            
            foreach (PlayerShot shot in playerShots)
            {
                shot.Show(false);
                shot.UnprepareItem();
            }
            foreach (PlayerShot shot in playerShots)
            {
                shot.Show(false);
                shot.UnprepareItem();
            }

            foreach (LargeExplosion largeExplosion in largeExplosions)
            {
                largeExplosion.Show(false);
                largeExplosion.UnprepareItem();
            }

            foreach (MediumExplosion mediumExplosion in mediumExplosions)
            {
                mediumExplosion.Show(false);
                mediumExplosion.UnprepareItem();
            }

            foreach (SmallExplosion smallExplosion in smallExplosions)
            {
                smallExplosion.Show(false);
                smallExplosion.UnprepareItem();

            }

            foreach (GroundEnemy groundEnemy in groundEnemyList)
            {
                groundEnemy.Unprepare();
            }
        }

        public void SubUpdate()
        {
            if (MyManager.GamePaused)
            {
                return;
            }

            if (MyGame.CurrentGameMode == GameMode.Ingame && MyManager.AllowUserInput)
            {
                if (shotFrameInterval == 0)
                {
                    if (MyManager.AllowUserInput && MyGame.FirePressed)
                    {
                        shotFrameInterval = maxShotFrameInterval;
                        SpawnPlayerShot(PlayerOffsetMapPosition, PlayerMapHeight);

                        if (MyGame.SfxOn)
                        {
                            MyGame.PlayerFireShotSound.Play();
                        }
                    }
                }
                else
                {
                    if (shotFrameInterval > 0)
                    {
                        shotFrameInterval--;
                    }
                }

                CheckPlayerMapCollision();
                HandleGroundEnemy();
            }

            UpdateMapBasedOnMapCenterPosition();
            UpdatePlayerShotpBasedOnMapCenterPosition();
            UpdateLargeExplosionBasedOnMapCenterPosition();
            UpdateMediumExplosionBasedOnMapCenterPosition();
            UpdateSmallExplosionBasedOnMapCenterPosition();
        }

        int ReadMapTileNr(int layer, int x, int y)
        {
            return mapBinaryData[layer][(y * MapWidth) + x];
        }

        void SetMapTileNr(int tileNr, int layer, int x, int y)
        {
            mapBinaryData[layer][(y * MapWidth) + x] = tileNr;
        }

        MapItemType ReadMapItemTypeFromTileNr(int layer, int x, int y)
        {
            int tileNr = ReadMapTileNr(layer, x, y);

            if (tileNr >= 0 && tileNr <= 63)
            {
                return MapItemType.Floor;
            }
            else if (tileNr >= 64 && tileNr <= 127)
            {
                return MapItemType.IndestructableTile;
            }
            else if (tileNr >= 128 && tileNr <= 191)
            {
                return MapItemType.EnemyStructure;
            }
            else if (tileNr >= 192 && tileNr <= 255)
            {
                return MapItemType.EnemyGround;
            }
            else if (tileNr >= 256 && tileNr <= 319)
            {
                return MapItemType.EnemyAir;
            }

            return MapItemType.Empty;
        }

        int FindNearestMapTileToPositionAndHeight(Vector2 position, float height, ref float finalMinDistance)
        {
            int nearestMapItemIndex = -1;
            float minDistance = int.MaxValue;
            float minHeightDifference = int.MaxValue;

            for (int index = 0; index < MapTileItems.Length; index++)
            {
                if (MapTileItems[index] == null)
                {
                    continue;
                }

                if (MapTileItems[index].Type == MapItemType.Empty)
                {
                    continue;
                }

                Vector2 distancePositionVector = MapTileItems[index].Position;
                distancePositionVector.X += 8;
                distancePositionVector.Y += 12;
                distancePositionVector -= position;

                float positionDistance = distancePositionVector.Length();
                float heightDifference = Math.Abs(MapTileItems[index].Height - height);

                if (positionDistance < minDistance)
                {
                    nearestMapItemIndex = index;
                    minDistance = positionDistance;
                    minHeightDifference = heightDifference;
                }
                else if (positionDistance == minDistance)
                {
                    if (heightDifference < minHeightDifference)
                    {
                        nearestMapItemIndex = index;
                        minDistance = positionDistance;
                        minHeightDifference = heightDifference;
                    }
                }
            }

            finalMinDistance = minDistance;

            return nearestMapItemIndex;
        }

        DisplayLayer FindTileLayerForHeight(float height)
        {
            DisplayLayer result = DisplayLayer.MapLayer0Tiles;

            int index = (int)(height / 7.0f);
            if (index >= 0 && index <= 4)
            {
                switch (index)
                {
                    case 0:
                        result = DisplayLayer.MapLayer0Tiles;
                        break;
                    case 1:
                        result = DisplayLayer.MapLayer1Tiles;
                        break;
                    case 2:
                        result = DisplayLayer.MapLayer2Tiles;
                        break;
                    case 3:
                        result = DisplayLayer.MapLayer3Tiles;
                        break;
                    case 4:
                        result = DisplayLayer.MapLayer4Tiles;
                        break;
                    default:
                        break;
                }
            }
            return result;
        }

        public void DestroyPlayer()
        {
            foreach (GroundEnemy enemy in groundEnemyList)
            {
                enemy.Unprepare();
            }

            SpawnLargeExplosion(PlayerOffsetMapPosition, PlayerMapHeight);

            if (MyGame.SfxOn)
            {
                MyGame.LargeHitSound.Play();
            }

            MyManager.PlayEndHoldSequence();
        }

        void CheckPlayerMapCollision()
        {
            float minDistance = 0.0f;
            int nearestTileIndex = FindNearestMapTileToPositionAndHeight(PlayerOffsetMapPosition, PlayerMapHeight, ref minDistance);

            if (nearestTileIndex != -1)
            {
                MapTileItem nearestTileItem = MapTileItems[nearestTileIndex];

                float heightDifference = heightDifference = Math.Abs(PlayerMapHeight - MapTileItems[nearestTileIndex].Height);
                if (MapTileItems[nearestTileIndex].Type != MapItemType.Floor && heightDifference <= 4.0f)
                {
                    DestroyPlayer();
                }
            }
        }

        bool LoadXmlFile(string filename)
        {
            doc = new XmlDocument();
            try
            {
                doc.Load(filename);
                isValidLevelLoaded = true;

                BuildMapBinaryData();

                RemoveAllMapSpritesFromComponents();
                BuildMapSprites();
                AddAllMapSpritesToComponents();

                ExtractEnemiesFromMap();

                return true;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                return false;
            }
            finally
            {
                doc = null;
            }
        }

        void BuildMapBinaryData()
        {
            mapBinaryData = new int[5][];

            if (!isValidLevelLoaded)
            {
                return;
            }

            XmlNodeList mapNodeList = doc.GetElementsByTagName("map");
            if (mapNodeList.Count == 0)
            {
                return;
            }

            XmlNodeList layerNodeList = doc.GetElementsByTagName("layer");
            if (layerNodeList.Count == 0)
            {
                return;
            }

            MapWidth = int.Parse(mapNodeList[0].Attributes.GetNamedItem("width").Value);
            MapHeight = int.Parse(mapNodeList[0].Attributes.GetNamedItem("height").Value);

            foreach (XmlNode layer in layerNodeList)
            {
                int layerIndex = int.Parse(layer.Attributes.GetNamedItem("name").Value);

                foreach (XmlNode childNode in layer.ChildNodes)
                {
                    if (childNode.Name == "data")
                    {
                        var dataEntriesLine = childNode.InnerText;
                        var dataLinesSplit = dataEntriesLine.Replace("\r\n", "").Split(new char[] { ',' });
                        var nosData = dataLinesSplit.Count();

                        int[] binaryLayerData = new int[nosData];
                        for (int i = 0; i < nosData; i++)
                        {
                            int origTileValue = int.Parse(dataLinesSplit[i]);
                            binaryLayerData[i] = origTileValue - 1;
                        }

                        mapBinaryData[layerIndex] = binaryLayerData;
                    }
                }
            }
        }

        void BuildMapSprites()
        {
            MapTileItems = new MapTileItem[5 * (MapWidth * MapHeight)];

            if (!isValidLevelLoaded)
            {
                return;
            }

            for (int layerIndex = 0; layerIndex <= 4; layerIndex++)
            {
                int newDisplayLayer = (int)DisplayLayer.MapLayer0Tiles;
                if (layerIndex == 1)
                {
                    newDisplayLayer = (int)DisplayLayer.MapLayer1Tiles;
                }
                else if (layerIndex == 2)
                {
                    newDisplayLayer = (int)DisplayLayer.MapLayer2Tiles;
                }
                else if (layerIndex == 3)
                {
                    newDisplayLayer = (int)DisplayLayer.MapLayer3Tiles;
                }
                else if (layerIndex == 4)
                {
                    newDisplayLayer = (int)DisplayLayer.MapLayer4Tiles;
                }

                for (int y = 0; y < MapHeight; y++)
                {
                    int moduloOffsetY = y % 2;

                    for (int x = 0; x < MapWidth; x++)
                    {
                        int tileNr = ReadMapTileNr(layerIndex, x, y);

                        int offset = (layerIndex * (MapWidth * MapHeight)) + (y * MapWidth) + x;

                        if (tileNr >= 0)
                        {
                            MapTileItems[offset] = new MapTileItem();

                            MapTileItems[offset].Type = ReadMapItemTypeFromTileNr(layerIndex, x, y);
                            MapTileItems[offset].TileNr = tileNr;

                            MapTileItems[offset].XCoordinate = x;
                            MapTileItems[offset].YCoordinate = y;
                            MapTileItems[offset].LayerIndex = layerIndex;

                            MapTileItems[offset].Position = new Vector2((x * 16.0f) + (moduloOffsetY * 8.0f), y * 4.0f);
                            MapTileItems[offset].Height = layerIndex * 7;
                            MapTileItems[offset].PixelSize = new int[2] { 16, 16 };
                            MapTileItems[offset].State = MapItemState.Alive;
                            MapTileItems[offset].HitPoints = 1;

                            MapTileItems[offset].Sprite = new SpriteGameVisual(MyGame);
                            MapTileItems[offset].Sprite.DrawOrder = newDisplayLayer;
                            MapTileItems[offset].Sprite.Enabled = false;
                            MapTileItems[offset].Sprite.UpdateOrder = (int)UpdateLayer.Last;
                            MapTileItems[offset].Sprite.SourceImage = MyGame.TilesTexture;
                            MapTileItems[offset].Sprite.SourceRectangle = new Rectangle((tileNr % 64) * 16, (tileNr / 64) * 16, 16, 16);
                            MapTileItems[offset].Sprite.TargetPosition = new Vector2(0.0f, 0.0f);
                            MapTileItems[offset].Sprite.TargetSize = new Vector2(16.0f, 16.0f);
                            MapTileItems[offset].Sprite.Visible = true;
                        }
                        else
                        {
                            MapTileItems[offset] = null;
                        }
                    }
                }
            }
        }

        void ExtractEnemiesFromMap()
        {
            if (groundEnemyList.Count > 0)
            {
                foreach (GroundEnemy enemy in groundEnemyList)
                {
                    enemy.Unprepare();
                }
            }
            groundEnemyList = new List<GroundEnemy>();

            for (int index = 0; index < MapTileItems.Length; index++)
            {
                MapTileItem mapTileItem = MapTileItems[index];

                if (mapTileItem == null)
                {
                    continue;
                }

                if (mapTileItem.Type == MapItemType.EnemyGround)
                {
                    GroundEnemy newEnemy = new GroundEnemy();
                    GroundEnemyType type = GroundEnemyType.None;
                    switch (mapTileItem.TileNr)
                    {
                        case 192:
                            type = GroundEnemyType.Turret1;

                            newEnemy.LastShotCounter = 0;
                            newEnemy.MaxShotCounter = 5 * 60;
                            newEnemy.ViewRange = 16.0f * 5.0f;
                            break;
                        case 193:
                            type = GroundEnemyType.Turret2;

                            newEnemy.LastShotCounter = 0;
                            newEnemy.MaxShotCounter = 5 * 60;
                            newEnemy.ViewRange = 16.0f * 5.0f;
                            break;
                        case 194:
                            type = GroundEnemyType.Turret3;

                            newEnemy.LastShotCounter = 0;
                            newEnemy.MaxShotCounter = 5 * 60;
                            newEnemy.ViewRange = 16.0f * 5.0f;
                            break;
                        case 195:
                            type = GroundEnemyType.Turret4;

                            newEnemy.LastShotCounter = 0;
                            newEnemy.MaxShotCounter = 5 * 60;
                            newEnemy.ViewRange = 16.0f * 5.0f;
                            break;
                        case 196:
                            type = GroundEnemyType.Bunker1;

                            newEnemy.LastShotCounter = 0;
                            newEnemy.MaxShotCounter = 5 * 60;
                            newEnemy.ViewRange = 16.0f * 7.0f;
                            break;
                        case 197:
                            type = GroundEnemyType.RocketLauncher1;

                            newEnemy.LastShotCounter = 0;
                            newEnemy.MaxShotCounter = 6 * 60;
                            newEnemy.ViewRange = 16.0f * 9.0f;
                            break;
                        default:
                            break;
                    }
                    newEnemy.Type = type;
                    newEnemy.mapTileItemIndex = index;
                    newEnemy.Prepare(this);

                    groundEnemyList.Add(newEnemy);
                }
            }
        }

        void DestroyGroundEnemy(int tileIndex)
        {
            for (int index = groundEnemyList.Count - 1; index >= 0; index--)
            {
                if (groundEnemyList[index].mapTileItemIndex == tileIndex)
                {
                    groundEnemyList[index].Unprepare();
                    groundEnemyList.RemoveAt(index);
                }
            }
        }

        void HandleGroundEnemy()
        {
            foreach (GroundEnemy enemy in groundEnemyList)
            {
                enemy.Think();
            }
        }

        void BuildCraterInMap(int index)
        {
            // change map type
            MapTileItems[index].Type = MapItemType.Floor;

            // Rebuild sprite
            MapTileItems[index].Sprite.DrawOrder = (int)DisplayLayer.MapLayer0Tiles;
            MapTileItems[index].Sprite.Enabled = false;
            MapTileItems[index].Sprite.UpdateOrder = (int)UpdateLayer.Last;
            MapTileItems[index].Sprite.SourceImage = MyGame.TilesTexture;
            MapTileItems[index].Sprite.SourceRectangle = new Rectangle((craterTileNr % 64) * 16, (craterTileNr / 64) * 16, 16, 16);
            MapTileItems[index].Sprite.TargetPosition = new Vector2(0.0f, 0.0f);
            MapTileItems[index].Sprite.TargetSize = new Vector2(16.0f, 16.0f);
            MapTileItems[index].Sprite.Visible = true;
        }

        public void AddAllMapSpritesToComponents()
        {
            if (MapTileItems == null)
            {
                return;
            }

            foreach (MapTileItem mapItem in MapTileItems)
            {
                if (mapItem != null && mapItem.Sprite != null)
                {
                    MyGame.Components.Add(mapItem.Sprite);
                }
            }
        }

        public void RemoveAllMapSpritesFromComponents()
        {
            if (MapTileItems == null)
            {
                return;
            }

            foreach (MapTileItem mapItem in MapTileItems)
            {
                if (mapItem != null && mapItem.Sprite != null)
                {
                    MyGame.Components.Remove(mapItem.Sprite);
                }
            }
        }

        public void ShowAllMapSprites(bool flag)
        {
            if (MapTileItems == null)
            {
                return;
            }

            foreach (MapTileItem mapItem in MapTileItems)
            {
                if (mapItem != null && mapItem.Sprite != null)
                {
                    if (mapItem.Sprite.Visible != flag)
                    {
                        mapItem.Sprite.Visible = flag;
                    }
                }
            }
        }

        public void UpdateMapBasedOnMapCenterPosition()
        {
            if (MyGame.CurrentGameMode == GameMode.Ingame)
            {
                if (!isValidLevelLoaded)
                {
                    return;
                }

                for (int layerIndex = 0; layerIndex <= 4; layerIndex++)
                {
                    for (int y = 0; y < MapHeight; y++)
                    {
                        int moduloOffsetY = y % 2;

                        for (int x = 0; x < MapWidth; x++)
                        {
                            int offset = (layerIndex * (MapWidth * MapHeight)) + (y * MapWidth) + x;
                            if (MapTileItems[offset] == null || MapTileItems[offset].Sprite == null)
                            {
                                continue;
                            }

                            MapTileItems[offset].UpdateBasedOnMapCenterPosition(PlayerMapPosition, MapScreenCenterAnchorPosition);
                        }
                    }
                }
            }
        }

        void SpawnPlayerShot(Vector2 mapPosition, float mapHeight)
        {
            PlayerShot newPlayerShot = new PlayerShot();
            newPlayerShot.PrepareItem(MyGame);
            newPlayerShot.SetPosition(mapPosition);
            newPlayerShot.SetHeight(mapHeight);
            newPlayerShot.IsPlaying = true;

            playerShots.Add(newPlayerShot);
        }

        void UpdatePlayerShotpBasedOnMapCenterPosition()
        {
            if (playerShots.Count <= 0)
            {
                return;
            }

            if (MyGame.CurrentGameMode == GameMode.Ingame)
            {
                for (int index = playerShots.Count - 1; index >= 0; index--)
                {
                    PlayerShot shot = playerShots[index];
                    if (shot.IsPlaying)
                    {
                        shot.SetPosition(shot.GetPosition() + (MyManager.UnitMapForward * PlayerShotSpeed));
                        shot.UpdateBasedOnMapCenterPosition(PlayerMapPosition, MapScreenCenterAnchorPosition);

                        shot.Show(true);

                        shot.LifetimeFrames--;
                        if (shot.LifetimeFrames <= 0)
                        {
                            shot.IsPlaying = false;
                            shot.UnprepareItem();

                            playerShots.RemoveAt(index);
                        }
                        else
                        {
                            float minDistance = int.MaxValue;
                            int nearestTileIndex = FindNearestMapTileToPositionAndHeight(shot.GetPosition(), shot.GetHeight(), ref minDistance);

                            if (nearestTileIndex != -1)
                            {
                                MapTileItem nearestTileItem = MapTileItems[nearestTileIndex];

                                float heightDifference = heightDifference = Math.Abs(shot.GetHeight() - MapTileItems[nearestTileIndex].Height);
                                if (heightDifference <= 7.0f)
                                {
                                    Vector2 explosionPosition = nearestTileItem.GetPosition();
                                    explosionPosition.X += 8;
                                    explosionPosition.Y += 12;
                                    float explosionHeight = nearestTileItem.GetHeight();

                                    if (nearestTileItem.Type == MapItemType.DestructableTile)
                                    {
                                        shot.LifetimeFrames = 0;

                                        SpawnMediumExplosion(explosionPosition, explosionHeight);

                                        if (nearestTileItem.LayerIndex == 0)
                                        {
                                            BuildCraterInMap(nearestTileIndex);
                                        }

                                        MyManager.RequestedPlayerScore += 10;
                                    }
                                    else if (nearestTileItem.Type == MapItemType.IndestructableTile)
                                    {
                                        shot.LifetimeFrames = 0;

                                        SpawnSmallExplosion(shot.GetPosition(), shot.GetHeight());

                                        if (MyGame.SfxOn)
                                        {
                                            MyGame.SmallHitSound.Play();
                                        }
                                    }
                                    else if (nearestTileItem.Type == MapItemType.EnemyStructure)
                                    {
                                        shot.LifetimeFrames = 0;

                                        SpawnLargeExplosion(explosionPosition, explosionHeight);

                                        if (MyGame.SfxOn)
                                        {
                                            MyGame.LargeHitSound.Play();
                                        }

                                        if (nearestTileItem.LayerIndex == 0)
                                        {
                                            BuildCraterInMap(nearestTileIndex);
                                        }

                                        MyManager.RequestedPlayerScore += 25;
                                    }
                                    else if (nearestTileItem.Type == MapItemType.EnemyGround)
                                    {
                                        shot.LifetimeFrames = 0;

                                        SpawnLargeExplosion(explosionPosition, explosionHeight);

                                        if (MyGame.SfxOn)
                                        {
                                            MyGame.LargeHitSound.Play();
                                        }

                                        if (nearestTileItem.LayerIndex == 0)
                                        {
                                            BuildCraterInMap(nearestTileIndex);
                                        }

                                        MyManager.RequestedPlayerScore += 25;

                                        DestroyGroundEnemy(nearestTileIndex);
                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        shot.Show(false);
                    }
                }
            }
            else
            {
                foreach (PlayerShot shot in playerShots)
                {
                    if (shot.IsPlaying)
                    {
                        shot.Show(false);
                    }
                }
            }
        }

        void SpawnLargeExplosion(Vector2 mapPosition, float mapHeight)
        {
            LargeExplosion newExplosion = new LargeExplosion();
            newExplosion.PrepareItem(MyGame);
            newExplosion.SetPosition(mapPosition);
            newExplosion.SetHeight(mapHeight);
            newExplosion.IsPlaying = true;

            largeExplosions.Add(newExplosion);
        }

        void UpdateLargeExplosionBasedOnMapCenterPosition()
        {
            if (largeExplosions.Count <= 0)
            {
                return;
            }

            if (MyGame.CurrentGameMode == GameMode.Ingame)
            {
                for (int index = largeExplosions.Count - 1; index >= 0; index--)
                {
                    LargeExplosion explosion = largeExplosions[index];
                    if (explosion.IsPlaying)
                    {
                        explosion.UpdateBasedOnMapCenterPosition(PlayerMapPosition, MapScreenCenterAnchorPosition);

                        explosion.Show(true);
                    }
                    else
                    {
                        explosion.Show(false);
                        explosion.UnprepareItem();

                        largeExplosions.RemoveAt(index);
                    }
                }
            }
            else
            {
                foreach (LargeExplosion explosion in largeExplosions)
                {
                    if (explosion.IsPlaying)
                    {
                        explosion.Show(false);
                    }
                }
            }

        }

        void SpawnMediumExplosion(Vector2 mapPosition, float mapHeight)
        {
            MediumExplosion newExplosion = new MediumExplosion();
            newExplosion.PrepareItem(MyGame);
            newExplosion.SetPosition(mapPosition);
            newExplosion.SetHeight(mapHeight);
            newExplosion.IsPlaying = true;

            mediumExplosions.Add(newExplosion);
        }

        void UpdateMediumExplosionBasedOnMapCenterPosition()
        {
            if (mediumExplosions.Count <= 0)
            {
                return;
            }

            if (MyGame.CurrentGameMode == GameMode.Ingame)
            {
                for (int index = mediumExplosions.Count - 1; index >= 0; index--)
                {
                    MediumExplosion explosion = mediumExplosions[index];
                    if (explosion.IsPlaying)
                    {
                        explosion.UpdateBasedOnMapCenterPosition(PlayerMapPosition, MapScreenCenterAnchorPosition);

                        explosion.Show(true);
                    }
                    else
                    {
                        explosion.Show(false);
                        explosion.UnprepareItem();

                        mediumExplosions.RemoveAt(index);
                    }
                }
            }
            else
            {
                foreach (MediumExplosion explosion in mediumExplosions)
                {
                    if (explosion.IsPlaying)
                    {
                        explosion.Show(false);
                    }
                }
            }
        }

        void SpawnSmallExplosion(Vector2 mapPosition, float mapHeight)
        {
            SmallExplosion newExplosion = new SmallExplosion();
            newExplosion.PrepareItem(MyGame);
            newExplosion.SetPosition(mapPosition);
            newExplosion.SetHeight(mapHeight);
            newExplosion.IsPlaying = true;

            smallExplosions.Add(newExplosion);
        }

        void UpdateSmallExplosionBasedOnMapCenterPosition()
        {
            if (smallExplosions.Count <= 0)
            {
                return;
            }

            if (MyGame.CurrentGameMode == GameMode.Ingame)
            {
                for (int index = smallExplosions.Count - 1; index >= 0; index--)
                {
                    SmallExplosion explosion = smallExplosions[index];
                    if (explosion.IsPlaying)
                    {
                        explosion.UpdateBasedOnMapCenterPosition(PlayerMapPosition, MapScreenCenterAnchorPosition);

                        explosion.Show(true);
                    }
                    else
                    {
                        explosion.Show(false);
                        explosion.UnprepareItem();

                        smallExplosions.RemoveAt(index);
                    }
                }
            }
            else
            {
                foreach (SmallExplosion explosion in smallExplosions)
                {
                    if (explosion.IsPlaying)
                    {
                        explosion.Show(false);
                    }
                }
            }
        }
    }
}
