﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using System.Collections.Generic;
using System.IO;

namespace Gbjam5MksGame
{
    public class MapManagerGameComponent : GameComponent
    {
        protected MyGame myGame;

        bool RequestReturnToTitleScreen = false;

        public bool AllowUserInput = false;
        public bool AllowMapScrolling = false;
        public bool InitiateRestartLevel = false;

        public bool GamePaused = false;
        public bool GamePausedReset = true;

        public int FrameCounter = 0;
        public int StartLevelFrameCounter;
        public int EndRestartLevelFrameCounter = 0;
        public int EndLevelOrGameActiveFrameCounter = 0;

        SpriteGameVisual playerSprite;
        SpriteGameVisual[] playerShadowSprites;
        SpriteGameVisual leftBarSprite;
        SpriteGameVisual rightBarSprite;
        SpriteGameVisual heightBarSprite;
        SpriteGameVisual goalBarSprite;
        SpriteGameVisual[] playerLifeSprites;
        SpriteGameVisual[] playerScoreSprites;
        SpriteGameVisual[] playerLevelSprites;

        SpriteGameVisual getReadySprite;
        SpriteGameVisual pausedSprite;
        SpriteGameVisual stageClearSprite;

        Vector2 playerLeftMostScreenAnchorPosition = new Vector2(11.0f, 105.0f);
        Vector2 playerScreenRange = new Vector2(74.0f, 37.0f);
        float playerLeftRightSpeed = 0.015f;
        float playerLeftRightFactor; // = 0.3f;
        public Vector2 PlayerScreenAnchorPosition = new Vector2(49.0f, 124.0f); // Interpolated L/R Map Position

        float playerClimbSinkSpeed = 0.35f;
        public float PlayerMinMapHeight = 0.0f;
        public float PlayerMaxMapHeight = 34.0f;
        Vector2 playerScreenDims = new Vector2(16.0f, 16.0f);

        float goalReachFactor; // = 0.0f;
        Vector2 levelStartPosition;
        Vector2 levelEndPosition;
        //float goalReachFactorIncrementPerFrame = 0.0000833f; = 1:12000
        float goalReachFactorIncrementPerFrame = 0.0001f; // 1:10000

        public int CurrentPlayerLevel = 0;
        public int MaxPlayerLevel = 5; //10; //15;

        public int CurrentPlayerLives; // = 3;

        public int CurrentPlayerScore; // = 0;
        public int RequestedPlayerScore; // = 0;
        int levelStartPlayerScore = 0;

        LevelManager levelManager = null;

        public Vector2 UnitMapForward;
        public Vector2 UnitMapRight;

        public MapManagerGameComponent(Game game) : base(game)
        {
            myGame = (MyGame)game;

            this.Enabled = true;
            this.UpdateOrder = (int)UpdateLayer.First;

            InitializeSprites();
        }

        public override void Update(GameTime gameTime)
        {
            if (myGame.CurrentGameMode == GameMode.Ingame)
            {
                if (CurrentPlayerScore > myGame.Hiscore)
                {
                    myGame.Hiscore = CurrentPlayerScore;
                }

                if (InitiateRestartLevel)
                {
                    InitiateRestartLevel = false;

                    RestartLevel();
                    return;
                }

                if (EndLevelOrGameActiveFrameCounter > 0)
                {
                    --EndLevelOrGameActiveFrameCounter;

                    if (EndLevelOrGameActiveFrameCounter == 0)
                    {
                        CurrentPlayerLevel++;
                        if (CurrentPlayerLevel <= MaxPlayerLevel)
                        {
                            InitiateRestartLevel = true;
                            return;
                        }
                        else
                        {
                            if (stageClearSprite.Visible)
                            {
                                stageClearSprite.Visible = false;
                            }

                            if (levelManager != null)
                            {
                                levelManager.SubUnprepare();
                            }

                            myGame.SwitchToGameWinScreen();
                            return;
                        }
                    }

                }
                else if (EndRestartLevelFrameCounter > 0)
                {
                    --EndRestartLevelFrameCounter;

                    if (EndRestartLevelFrameCounter == 0)
                    {
                        RequestedPlayerScore = levelStartPlayerScore;
                        CurrentPlayerScore = levelStartPlayerScore;

                        CurrentPlayerLives--;
                        if (CurrentPlayerLives > 0)
                        {
                            InitiateRestartLevel = true;
                            return;
                        }
                        else
                        {
                            if (levelManager != null)
                            {
                                levelManager.SubUnprepare();
                            }

                            myGame.SwitchToGameOverScreen();
                            return;
                        }
                    }
                }
                else if (StartLevelFrameCounter == 0)
                {
                    if (!AllowUserInput)
                    {
                        AllowUserInput = true;
                    }

                    if (GamePausedReset && myGame.PausePressed)
                    {
                        GamePaused = !GamePaused;
                        GamePausedReset = false;

                        if (myGame.SfxOn)
                        {
                            myGame.MenuSelectSound.Play();
                        }

                        if (GamePaused)
                        {
                            if (!pausedSprite.Visible)
                            {
                                pausedSprite.Visible = true;
                            }
                        }
                        else
                        {
                            if (pausedSprite.Visible)
                            {
                                pausedSprite.Visible = false;
                            }
                        }
                    }
                    else if (!GamePausedReset && !myGame.PausePressed)
                    {
                        GamePausedReset = true;
                    }

                    if (GamePaused)
                    {
                        if (RequestReturnToTitleScreen && !myGame.BackPressed)
                        {
                            RequestReturnToTitleScreen = false;

                            if (pausedSprite.Visible)
                            {
                                pausedSprite.Visible = false;
                            }

                            if(levelManager != null)
                            {
                                levelManager.SubUnprepare();
                            }

                            if (myGame.SfxOn)
                            {
                                myGame.MenuSelectSound.Play();
                            }

                            myGame.SwitchToTitleScreen();
                            return;
                        }
                        else if (!RequestReturnToTitleScreen && myGame.BackPressed)
                        {
                            RequestReturnToTitleScreen = true;
                        }
                    }
                    else if (!GamePaused)
                    {
                        if (myGame.MoveUpPressed)
                        {
                            levelManager.PlayerMapHeight += playerClimbSinkSpeed;
                            if (levelManager.PlayerMapHeight > PlayerMaxMapHeight)
                            {
                                levelManager.PlayerMapHeight = PlayerMaxMapHeight;
                            }
                        }
                        else if (myGame.MoveDownPressed)
                        {
                            levelManager.PlayerMapHeight -= playerClimbSinkSpeed;
                            if (levelManager.PlayerMapHeight < PlayerMinMapHeight)
                            {
                                levelManager.PlayerMapHeight = PlayerMinMapHeight;
                            }
                        }

                        if (myGame.MoveLeftPressed)
                        {
                            playerLeftRightFactor -= playerLeftRightSpeed;
                            if (playerLeftRightFactor < 0.0f)
                            {
                                playerLeftRightFactor = 0.0f;
                            }
                        }
                        else if (myGame.MoveRightPressed)
                        {
                            playerLeftRightFactor += playerLeftRightSpeed;
                            if (playerLeftRightFactor > 1.0f)
                            {
                                playerLeftRightFactor = 1.0f;
                            }
                        }

                        if (AllowMapScrolling)
                        {
                            goalReachFactor += goalReachFactorIncrementPerFrame;
                            if (goalReachFactor >= 1.0f)
                            {
                                goalReachFactor = 1.0f;

                                AllowUserInput = false;

                                EndLevelOrGameActiveFrameCounter = 5 * 60;

                                if (!stageClearSprite.Visible)
                                {
                                    stageClearSprite.Visible = true;
                                }
                            }
                        }

                        if (getReadySprite.Visible)
                        {
                            getReadySprite.Visible = false;
                        }
                    }
                }
                else
                {
                    --StartLevelFrameCounter;

                    int moduloBlink = FrameCounter % 60;
                    if (moduloBlink < 30 && !getReadySprite.Visible)
                    {
                        getReadySprite.Visible = true;
                    }
                    else if (moduloBlink >= 30 && getReadySprite.Visible)
                    {
                        getReadySprite.Visible = false;
                    }
                }

                ShowSpritesForIngameMode(true);
                ShowTilesForIngameMode(true);

                InterpolatePlayerMapPosition();

                UpdatePlayerAndShadowDisplay();
                UpdatePlayerHeightBarDisplay();
                UpdateGoalReachFactorDisplay();
                UpdatePlayerScoreDisplay();
                UpdatePlayerLivesDisplay();
                UpdatePlayerLevelDisplay();

                if (CurrentPlayerScore < RequestedPlayerScore)
                {
                    CurrentPlayerScore++;
                }

                FrameCounter++;
                while (FrameCounter >= 1000)
                    FrameCounter -= 1000;
            }
            else
            {
                ShowSpritesForIngameMode(false);
                ShowTilesForIngameMode(false);
            }

            if (levelManager != null)
            {
                levelManager.SubUpdate();
            }

            base.Update(gameTime);
        }

        void InterpolatePlayerMapPosition()
        {
            levelManager.PlayerMapPosition.X = levelStartPosition.X + ((levelEndPosition.X - levelStartPosition.X) * goalReachFactor);
            levelManager.PlayerMapPosition.Y = levelStartPosition.Y + ((levelEndPosition.Y - levelStartPosition.Y) * goalReachFactor);

            PlayerScreenAnchorPosition.X = playerLeftMostScreenAnchorPosition.X + (playerLeftRightFactor * playerScreenRange.X);
            PlayerScreenAnchorPosition.Y = playerLeftMostScreenAnchorPosition.Y + (playerLeftRightFactor * playerScreenRange.Y);

            levelManager.PlayerOffsetMapPosition = levelManager.PlayerMapPosition - (levelManager.MapScreenCenterAnchorPosition - PlayerScreenAnchorPosition);
        }

        void ShowSpritesForIngameMode(bool flag)
        {
            if (playerSprite.Visible != flag)
            {
                playerSprite.Visible = flag;
            }

            foreach (SpriteGameVisual sprite in playerShadowSprites)
            {
                if (sprite.Visible != flag)
                {
                    sprite.Visible = flag;
                }
            }

            if (leftBarSprite.Visible != flag)
            {
                leftBarSprite.Visible = flag;
            }

            if (rightBarSprite.Visible != flag)
            {
                rightBarSprite.Visible = flag;
            }

            if (heightBarSprite.Visible != flag)
            {
                heightBarSprite.Visible = flag;
            }

            if (goalBarSprite.Visible != flag)
            {
                goalBarSprite.Visible = flag;
            }

            for (int lifeIndex = 0; lifeIndex <= 2; lifeIndex++)
            {
                if (playerLifeSprites[lifeIndex].Visible != flag)
                {
                    playerLifeSprites[lifeIndex].Visible = flag;
                }
            }

            for (int scoreIndex = 0; scoreIndex <= 6; scoreIndex++)
            {
                if (playerScoreSprites[scoreIndex].Visible != flag)
                {
                    playerScoreSprites[scoreIndex].Visible = flag;
                }
            }

            for (int levelIndex = 0; levelIndex <= 1; levelIndex++)
            {
                if (playerLevelSprites[levelIndex].Visible != flag)
                {
                    playerLevelSprites[levelIndex].Visible = flag;
                }
            }
        }

        void ShowTilesForIngameMode(bool flag)
        {
            if (levelManager != null)
            {
                levelManager.ShowAllMapSprites(flag);
            }
        }

        void UpdatePlayerAndShadowDisplay()
        {
            playerSprite.TargetPosition.X = PlayerScreenAnchorPosition.X - 8.0f;
            playerSprite.TargetPosition.Y = PlayerScreenAnchorPosition.Y - levelManager.PlayerMapHeight - 8.0f;

            int displayLayerOrderIndex = ((int)levelManager.PlayerMapHeight / 7);
            if (displayLayerOrderIndex == 0)
            {
                playerSprite.DrawOrder = (int)DisplayLayer.MapLayer0Sprites;
            }
            else if (displayLayerOrderIndex == 1)
            {
                playerSprite.DrawOrder = (int)DisplayLayer.MapLayer1Sprites;
            }
            else if (displayLayerOrderIndex == 2)
            {
                playerSprite.DrawOrder = (int)DisplayLayer.MapLayer2Sprites;
            }
            else if (displayLayerOrderIndex == 3)
            {
                playerSprite.DrawOrder = (int)DisplayLayer.MapLayer3Sprites;
            }
            else if (displayLayerOrderIndex == 4)
            {
                playerSprite.DrawOrder = (int)DisplayLayer.MapLayer4Sprites;
            }

            playerShadowSprites[0].TargetPosition = new Vector2(PlayerScreenAnchorPosition.X - 8.0f, PlayerScreenAnchorPosition.Y - 8.0f);
            playerShadowSprites[1].TargetPosition = new Vector2(PlayerScreenAnchorPosition.X - 8.0f, PlayerScreenAnchorPosition.Y - 8.0f);
            playerShadowSprites[2].TargetPosition = new Vector2(PlayerScreenAnchorPosition.X - 8.0f, PlayerScreenAnchorPosition.Y - 8.0f);
            playerShadowSprites[3].TargetPosition = new Vector2(PlayerScreenAnchorPosition.X - 8.0f, PlayerScreenAnchorPosition.Y - 8.0f);
            playerShadowSprites[4].TargetPosition = new Vector2(PlayerScreenAnchorPosition.X - 8.0f, PlayerScreenAnchorPosition.Y - 8.0f);

            // Update visibility of correct shadow
            playerShadowSprites[0].Visible = false;
            playerShadowSprites[1].Visible = false;
            playerShadowSprites[2].Visible = false;
            playerShadowSprites[3].Visible = false;
            playerShadowSprites[4].Visible = false;
            int shadowIndex = (int)(levelManager.PlayerMapHeight / 7.0f);
            playerShadowSprites[shadowIndex].Visible = true;

            // TODO - Update layer for shadow based on height of non-empty map tiles under the player ship
        }

        void UpdatePlayerHeightBarDisplay()
        {
            int heightBarRatio = (int)levelManager.PlayerMapHeight / (int)PlayerMaxMapHeight - (int)PlayerMinMapHeight;
            int reqBarHeight = (80 * (int)levelManager.PlayerMapHeight) / ((int)PlayerMaxMapHeight - (int)PlayerMinMapHeight);

            heightBarSprite.TargetPosition.Y = (float)(112 - reqBarHeight);
            heightBarSprite.TargetSize.Y = (float)reqBarHeight;
        }

        void UpdateGoalReachFactorDisplay()
        {
            goalBarSprite.TargetPosition.Y = 109.0f - (goalReachFactor * 77.0f);
        }

        void UpdatePlayerScoreDisplay()
        {
            int currentRemain = CurrentPlayerScore;

            int digit0 = currentRemain / 1000000;
            currentRemain -= digit0 * 1000000;

            int digit1 = currentRemain / 100000;
            currentRemain -= digit1 * 100000;

            int digit2 = currentRemain / 10000;
            currentRemain -= digit2 * 10000;

            int digit3 = currentRemain / 1000;
            currentRemain -= digit3 * 1000;

            int digit4 = currentRemain / 100;
            currentRemain -= digit4 * 100;

            int digit5 = currentRemain / 10;
            currentRemain -= digit5 * 10;

            int digit6 = currentRemain;

            playerScoreSprites[0].SourceRectangle.X = 48 + (digit0 * 8);
            playerScoreSprites[1].SourceRectangle.X = 48 + (digit1 * 8);
            playerScoreSprites[2].SourceRectangle.X = 48 + (digit2 * 8);
            playerScoreSprites[3].SourceRectangle.X = 48 + (digit3 * 8);
            playerScoreSprites[4].SourceRectangle.X = 48 + (digit4 * 8);
            playerScoreSprites[5].SourceRectangle.X = 48 + (digit5 * 8);
            playerScoreSprites[6].SourceRectangle.X = 48 + (digit6 * 8);
        }

        void UpdatePlayerLivesDisplay()
        {
            playerLifeSprites[2].Visible = CurrentPlayerLives >= 3 ? true : false;
            playerLifeSprites[1].Visible = CurrentPlayerLives >= 2 ? true : false;
            playerLifeSprites[0].Visible = CurrentPlayerLives >= 1 ? true : false;
        }

        void UpdatePlayerLevelDisplay()
        {
            int currentRemain = CurrentPlayerLevel;

            int digit0 = currentRemain / 10;
            currentRemain -= digit0 * 10;

            int digit1 = currentRemain;

            playerLevelSprites[0].SourceRectangle.X = 48 + (digit0 * 8);
            playerLevelSprites[1].SourceRectangle.X = 48 + (digit1 * 8);
        }

        void InitializeSprites()
        {
            #region Initialize Sprites

            playerSprite = new SpriteGameVisual(myGame);
            playerSprite.DrawOrder = (int)DisplayLayer.MapLayer0Sprites;
            playerSprite.Enabled = false;
            playerSprite.UpdateOrder = (int)UpdateLayer.Last;
            playerSprite.SourceImage = myGame.TilesTexture;
            playerSprite.SourceRectangle = new Rectangle(0, 320, 16, 16);
            playerSprite.TargetPosition = new Vector2(0.0f, 0.0f);
            playerSprite.TargetSize = new Vector2(16.0f, 16.0f);
            myGame.Components.Add(playerSprite);

            playerShadowSprites = new SpriteGameVisual[5];

            playerShadowSprites[0] = new SpriteGameVisual(myGame);
            playerShadowSprites[0].DrawOrder = (int)DisplayLayer.MapLayer0Shadow;
            playerShadowSprites[0].Enabled = false;
            playerShadowSprites[0].UpdateOrder = (int)UpdateLayer.Last;
            playerShadowSprites[0].SourceImage = myGame.TilesTexture;
            playerShadowSprites[0].SourceRectangle = new Rectangle(0, 336, 16, 16);
            playerShadowSprites[0].TargetPosition = new Vector2(0.0f, 0.0f);
            playerShadowSprites[0].TargetSize = new Vector2(16.0f, 16.0f);
            myGame.Components.Add(playerShadowSprites[0]);

            playerShadowSprites[1] = new SpriteGameVisual(myGame);
            playerShadowSprites[1].DrawOrder = (int)DisplayLayer.MapLayer0Shadow;
            playerShadowSprites[1].Enabled = false;
            playerShadowSprites[1].UpdateOrder = (int)UpdateLayer.Last;
            playerShadowSprites[1].SourceImage = myGame.TilesTexture;
            playerShadowSprites[1].SourceRectangle = new Rectangle(16, 336, 16, 16);
            playerShadowSprites[1].TargetPosition = new Vector2(0.0f, 0.0f);
            playerShadowSprites[1].TargetSize = new Vector2(16.0f, 16.0f);
            myGame.Components.Add(playerShadowSprites[1]);

            playerShadowSprites[2] = new SpriteGameVisual(myGame);
            playerShadowSprites[2].DrawOrder = (int)DisplayLayer.MapLayer0Shadow;
            playerShadowSprites[2].Enabled = false;
            playerShadowSprites[2].UpdateOrder = (int)UpdateLayer.Last;
            playerShadowSprites[2].SourceImage = myGame.TilesTexture;
            playerShadowSprites[2].SourceRectangle = new Rectangle(32, 336, 16, 16);
            playerShadowSprites[2].TargetPosition = new Vector2(0.0f, 0.0f);
            playerShadowSprites[2].TargetSize = new Vector2(16.0f, 16.0f);
            myGame.Components.Add(playerShadowSprites[2]);

            playerShadowSprites[3] = new SpriteGameVisual(myGame);
            playerShadowSprites[3].DrawOrder = (int)DisplayLayer.MapLayer0Shadow;
            playerShadowSprites[3].Enabled = false;
            playerShadowSprites[3].UpdateOrder = (int)UpdateLayer.Last;
            playerShadowSprites[3].SourceImage = myGame.TilesTexture;
            playerShadowSprites[3].SourceRectangle = new Rectangle(48, 336, 16, 16);
            playerShadowSprites[3].TargetPosition = new Vector2(0.0f, 0.0f);
            playerShadowSprites[3].TargetSize = new Vector2(16.0f, 16.0f);
            myGame.Components.Add(playerShadowSprites[3]);

            playerShadowSprites[4] = new SpriteGameVisual(myGame);
            playerShadowSprites[4].DrawOrder = (int)DisplayLayer.MapLayer0Shadow;
            playerShadowSprites[4].Enabled = false;
            playerShadowSprites[4].UpdateOrder = (int)UpdateLayer.Last;
            playerShadowSprites[4].SourceImage = myGame.TilesTexture;
            playerShadowSprites[4].SourceRectangle = new Rectangle(64, 336, 16, 16);
            playerShadowSprites[4].TargetPosition = new Vector2(0.0f, 0.0f);
            playerShadowSprites[4].TargetSize = new Vector2(16.0f, 16.0f);
            myGame.Components.Add(playerShadowSprites[4]);

            leftBarSprite = new SpriteGameVisual(myGame);
            leftBarSprite.DrawOrder = (int)DisplayLayer.MapForeground;
            leftBarSprite.Enabled = false;
            leftBarSprite.UpdateOrder = (int)UpdateLayer.Last;
            leftBarSprite.SourceImage = myGame.IngameScreenTexture;
            leftBarSprite.SourceRectangle = new Rectangle(0, 0, 8, 144);
            leftBarSprite.TargetPosition = new Vector2(0.0f, 0.0f);
            leftBarSprite.TargetSize = new Vector2(8.0f, 144.0f);
            myGame.Components.Add(leftBarSprite);

            rightBarSprite = new SpriteGameVisual(myGame);
            rightBarSprite.DrawOrder = (int)DisplayLayer.MapForeground;
            rightBarSprite.Enabled = false;
            rightBarSprite.UpdateOrder = (int)UpdateLayer.Last;
            rightBarSprite.SourceImage = myGame.IngameScreenTexture;
            rightBarSprite.SourceRectangle = new Rectangle(152, 0, 8, 144);
            rightBarSprite.TargetPosition = new Vector2(152.0f, 0.0f);
            rightBarSprite.TargetSize = new Vector2(8.0f, 144.0f);
            myGame.Components.Add(rightBarSprite);

            heightBarSprite = new SpriteGameVisual(myGame);
            heightBarSprite.DrawOrder = (int)DisplayLayer.MapForeground;
            heightBarSprite.Enabled = false;
            heightBarSprite.UpdateOrder = (int)UpdateLayer.Last;
            heightBarSprite.SourceImage = myGame.TilesTexture;
            heightBarSprite.SourceRectangle = new Rectangle(1, 240, 6, 80);
            heightBarSprite.TargetPosition = new Vector2(1.0f, 32.0f);
            heightBarSprite.TargetSize = new Vector2(6.0f, 80.0f);
            myGame.Components.Add(heightBarSprite);

            goalBarSprite = new SpriteGameVisual(myGame);
            goalBarSprite.DrawOrder = (int)DisplayLayer.MapForeground;
            goalBarSprite.Enabled = false;
            goalBarSprite.UpdateOrder = (int)UpdateLayer.Last;
            goalBarSprite.SourceImage = myGame.TilesTexture;
            goalBarSprite.SourceRectangle = new Rectangle(15, 240, 6, 3);
            goalBarSprite.TargetPosition = new Vector2(153.0f, 109.0f);
            goalBarSprite.TargetSize = new Vector2(6.0f, 3.0f);
            myGame.Components.Add(goalBarSprite);

            playerLifeSprites = new SpriteGameVisual[3];
            for (int lifeIndex = 0; lifeIndex <= 2; lifeIndex++)
            {
                playerLifeSprites[lifeIndex] = new SpriteGameVisual(myGame);
                playerLifeSprites[lifeIndex].DrawOrder = (int)DisplayLayer.UI;
                playerLifeSprites[lifeIndex].Enabled = false;
                playerLifeSprites[lifeIndex].UpdateOrder = (int)UpdateLayer.Last;
                playerLifeSprites[lifeIndex].SourceImage = myGame.TilesTexture;
                playerLifeSprites[lifeIndex].SourceRectangle = new Rectangle(32, 240, 8, 8);
                playerLifeSprites[lifeIndex].TargetPosition = new Vector2(12.0f + (lifeIndex * 8.0f), 14.0f);
                playerLifeSprites[lifeIndex].TargetSize = new Vector2(8.0f, 8.0f);
                myGame.Components.Add(playerLifeSprites[lifeIndex]);
            }

            playerScoreSprites = new SpriteGameVisual[7];
            for (int scoreIndex = 0; scoreIndex <= 6; scoreIndex++)
            {
                playerScoreSprites[scoreIndex] = new SpriteGameVisual(myGame);
                playerScoreSprites[scoreIndex].DrawOrder = (int)DisplayLayer.UI;
                playerScoreSprites[scoreIndex].Enabled = false;
                playerScoreSprites[scoreIndex].UpdateOrder = (int)UpdateLayer.Last;
                playerScoreSprites[scoreIndex].SourceImage = myGame.TilesTexture;
                playerScoreSprites[scoreIndex].SourceRectangle = new Rectangle(48, 240, 8, 8);
                playerScoreSprites[scoreIndex].TargetPosition = new Vector2(12.0f + (scoreIndex * 8.0f), 4.0f);
                playerScoreSprites[scoreIndex].TargetSize = new Vector2(8.0f, 8.0f);
                myGame.Components.Add(playerScoreSprites[scoreIndex]);
            }

            playerLevelSprites = new SpriteGameVisual[7];
            for (int levelIndex = 0; levelIndex <= 1; levelIndex++)
            {
                playerLevelSprites[levelIndex] = new SpriteGameVisual(myGame);
                playerLevelSprites[levelIndex].DrawOrder = (int)DisplayLayer.UI;
                playerLevelSprites[levelIndex].Enabled = false;
                playerLevelSprites[levelIndex].UpdateOrder = (int)UpdateLayer.Last;
                playerLevelSprites[levelIndex].SourceImage = myGame.TilesTexture;
                playerLevelSprites[levelIndex].SourceRectangle = new Rectangle(48, 240, 8, 8);
                playerLevelSprites[levelIndex].TargetPosition = new Vector2(130.0f + (levelIndex * 8.0f), 132.0f);
                playerLevelSprites[levelIndex].TargetSize = new Vector2(8.0f, 8.0f);
                myGame.Components.Add(playerLevelSprites[levelIndex]);
            }

            getReadySprite = new SpriteGameVisual(myGame);
            getReadySprite.DrawOrder = (int)DisplayLayer.MapForeground;
            getReadySprite.Enabled = false;
            getReadySprite.UpdateOrder = (int)UpdateLayer.Last;
            getReadySprite.SourceImage = myGame.TilesTexture;
            getReadySprite.SourceRectangle = new Rectangle(32, 256, 80, 32);
            getReadySprite.TargetPosition = new Vector2(40.0f, 56.0f);
            getReadySprite.TargetSize = new Vector2(80.0f, 32.0f);
            getReadySprite.Visible = false; // initially hidden
            myGame.Components.Add(getReadySprite);

            pausedSprite = new SpriteGameVisual(myGame);
            pausedSprite.DrawOrder = (int)DisplayLayer.MapForeground;
            pausedSprite.Enabled = false;
            pausedSprite.UpdateOrder = (int)UpdateLayer.Last;
            pausedSprite.SourceImage = myGame.TilesTexture;
            pausedSprite.SourceRectangle = new Rectangle(112, 256, 96, 32);
            pausedSprite.TargetPosition = new Vector2(32.0f, 56.0f);
            pausedSprite.TargetSize = new Vector2(96.0f, 32.0f);
            pausedSprite.Visible = false; // initially hidden
            myGame.Components.Add(pausedSprite);

            stageClearSprite = new SpriteGameVisual(myGame);
            stageClearSprite.DrawOrder = (int)DisplayLayer.MapForeground;
            stageClearSprite.Enabled = false;
            stageClearSprite.UpdateOrder = (int)UpdateLayer.Last;
            stageClearSprite.SourceImage = myGame.TilesTexture;
            stageClearSprite.SourceRectangle = new Rectangle(112, 288, 84, 9);
            stageClearSprite.TargetPosition = new Vector2(38.0f, 56.0f);
            stageClearSprite.TargetSize = new Vector2(84.0f, 9.0f);
            stageClearSprite.Visible = false; // initially hidden
            myGame.Components.Add(stageClearSprite);

            #endregion
        }

        public void PrepareNewGame()
        {
            CurrentPlayerLevel = 1;
            CurrentPlayerLives = 3;

            CurrentPlayerScore = 0;
            RequestedPlayerScore = 0;

            RestartLevel();
        }

        void RestartLevel()
        {
            levelStartPlayerScore = CurrentPlayerScore;

            //string cwd = Directory.GetCurrentDirectory();
            if (levelManager != null)
            {
                levelManager.RemoveAllMapSpritesFromComponents();
                levelManager = null;
            }
            levelManager = new LevelManager();
            levelManager.SubPrepare(myGame, this, CurrentPlayerLevel);
            if (levelManager.IsValidLevelLoaded)
            {
                levelStartPosition = new Vector2(40.0f, (levelManager.MapHeight * 4.0f) - 10.0f);
                levelEndPosition = new Vector2((levelManager.MapWidth * 16.0f) - 33.0f, 27.0f);

                levelManager.PlayerMapPosition = levelStartPosition;
            }
            levelManager.PlayerMapHeight = ((PlayerMaxMapHeight - PlayerMinMapHeight) / 2) + PlayerMinMapHeight;

            goalReachFactor = 0.0f; // 0.95f; // 0.0f;
            AllowMapScrolling = true;

            playerLeftRightFactor = 0.5f;

            AllowUserInput = false;
            StartLevelFrameCounter = 5 * 60;
            EndRestartLevelFrameCounter = 0;
            EndLevelOrGameActiveFrameCounter = 0;

            UnitMapForward = levelEndPosition - levelStartPosition;
            UnitMapForward.Normalize();
            UnitMapRight = playerScreenRange;
            UnitMapRight.Normalize();

            GamePaused = false;

            if (pausedSprite.Visible)
            {
                pausedSprite.Visible = false;
            }

            if (stageClearSprite.Visible)
            {
                stageClearSprite.Visible = false;
            }
        }

        public void PlayEndHoldSequence()
        {
            AllowUserInput = false;
            EndRestartLevelFrameCounter = 3 * 60;
        }

        public float GetLevelProgressFactor()
        {
            return (float)CurrentPlayerLevel / (float)MaxPlayerLevel;
        }

        public float GetLevelProgressSpeedFactor()
        {
            return 0.5f + GetLevelProgressFactor();
        }

    }
}
