﻿using Microsoft.Xna.Framework;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Gbjam5MksGame
{
    public enum MapItemType { Empty, Floor, IndestructableTile, DestructableTile, EnemyStructure, EnemyGround, EnemyAir, PlayerShot, EnemyMissile, EnemyShot, Explosion };
    public enum MapItemState { None, Dead, Alive, Spawning, Dying }

    public class MapTileItem
    {
        public SpriteGameVisual Sprite;
        public Vector2 Position;
        public float Height;
        public int[] PixelSize;
        public MapItemType Type;
        public int TileNr;
        public MapItemState State;
        public int HitPoints;

        public int XCoordinate;
        public int YCoordinate;
        public int LayerIndex;

        public MapTileItem()
        {
            Sprite = null;
            Position = new Vector2(0.0f, 0.0f);
            Height = 0.0f;
            PixelSize = new int[2] { 0, 0 };
            Type = MapItemType.Empty;
            TileNr = -1;
            State = MapItemState.None;
            HitPoints = 1;

            XCoordinate = 0;
            YCoordinate = 0;
            LayerIndex = 0;
        }

        public void SetPosition(Vector2 newPosition)
        {
            Position = newPosition;
        }

        public Vector2 GetPosition()
        {
            return Position;
        }

        public void SetHeight(float newHeight)
        {
            Height = newHeight;

            if (Sprite != null)
            {
                Sprite.DrawOrder = (int)FindTileLayerForHeight(Height);
            }
        }

        public float GetHeight()
        {
            return Height;
        }

        public DisplayLayer GetDrawOrder()
        {            
            if (Sprite != null)
            {
                return (DisplayLayer)Sprite.DrawOrder;
            }

            return DisplayLayer.MapLayer0Tiles;
        }

        public void UpdateBasedOnMapCenterPosition(Vector2 mapCenterPosition, Vector2 screenAnchorPosition)
        {
            if (Sprite != null)
            {
                Sprite.TargetPosition.X = Position.X - mapCenterPosition.X - (Sprite.TargetSize.X / 2) + 8;
                Sprite.TargetPosition.Y = Position.Y - mapCenterPosition.Y - (Sprite.TargetSize.Y / 2) - Height + 12;

                Sprite.TargetPosition.X += screenAnchorPosition.X;
                Sprite.TargetPosition.Y += screenAnchorPosition.Y;
            }
        }

        DisplayLayer FindTileLayerForHeight(float height)
        {
            DisplayLayer result = DisplayLayer.MapLayer0Tiles;

            int index = (int)(height / 7.0f);
            if (index >= 0 && index <= 4)
            {
                switch (index)
                {
                    case 0:
                        result = DisplayLayer.MapLayer0Tiles;
                        break;
                    case 1:
                        result = DisplayLayer.MapLayer1Tiles;
                        break;
                    case 2:
                        result = DisplayLayer.MapLayer2Tiles;
                        break;
                    case 3:
                        result = DisplayLayer.MapLayer3Tiles;
                        break;
                    case 4:
                        result = DisplayLayer.MapLayer4Tiles;
                        break;
                    default:
                        break;
                }
            }

            return result;
        }
    }
}
