﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using System.Collections.Generic;

namespace Gbjam5MksGame
{
    class UiManagerGameComponent : GameComponent
    {
        protected MyGame myGame;

        enum KeysPressedLast
        {
            None,
            MoveUpPressedLast, MoveDownPressedLast, MoveLeftPressedLast, MoveRightPressedLast,
            FirePressedLast, PausePressedLast, BackPressedLast
        }

        KeysPressedLast lastMenuInputKey = KeysPressedLast.None;
        bool lastMenuInputReleased = true;

        // MAIN MENU - Start, Help, Configure, QuitGame
        int mainMenuHoveredIndex = 0;
        SpriteGameVisual mainMenuStartSpriteNormal;
        SpriteGameVisual mainMenuStartSpriteHovered;
        SpriteGameVisual mainMenuHelpSpriteNormal;
        SpriteGameVisual mainMenuHelpSpriteHovered;
        SpriteGameVisual mainMenuConfigureSpriteNormal;
        SpriteGameVisual mainMenuConfigureSpriteHovered;
        SpriteGameVisual mainMenuQuitSpriteNormal;
        SpriteGameVisual mainMenuQuitSpriteHovered;

        // HELP MENU - Back
        SpriteGameVisual helpMenuBackSpriteHovered;

        // GAME OVER MENU - Back
        SpriteGameVisual gameOverMenuBackSpriteHovered;

        // GAME WIN MENU - Back
        SpriteGameVisual gameWinMenuBackSpriteHovered;

        // CONFIGURE MENU - Music ON/OFF, Sound FX On/OFF, Back button part of general handling
        int configureMenuHoveredIndex = 2;
        SpriteGameVisual configureMenuSfxToggleOffSpriteNormal;
        SpriteGameVisual configureMenuSfxToggleOffSpriteHovered;
        SpriteGameVisual configureMenuSfxToggleOnSpriteNormal;
        SpriteGameVisual configureMenuSfxToggleOnSpriteHovered;
        SpriteGameVisual configureMenuMusicToggleOffSpriteNormal;
        SpriteGameVisual configureMenuMusicToggleOffSpriteHovered;
        SpriteGameVisual configureMenuMusicToggleOnSpriteNormal;
        SpriteGameVisual configureMenuMusicToggleOnSpriteHovered;
        SpriteGameVisual configureMenuBackSpriteNormal;
        SpriteGameVisual configureMenuBackSpriteHovered;

        public UiManagerGameComponent(Game game) : base(game)
        {
            myGame = (MyGame)game;

            this.Enabled = true;
            this.UpdateOrder = (int)UpdateLayer.First;

            #region Initialize Sprites

            // Main
            mainMenuStartSpriteNormal = new SpriteGameVisual(game);
            mainMenuStartSpriteNormal.DrawOrder = (int)DisplayLayer.UI;
            mainMenuStartSpriteNormal.Enabled = false;
            mainMenuStartSpriteNormal.UpdateOrder = (int)UpdateLayer.Last;
            mainMenuStartSpriteNormal.SourceImage = myGame.TilesTexture;
            mainMenuStartSpriteNormal.SourceRectangle = new Rectangle(0, 208, 16, 13);
            mainMenuStartSpriteNormal.TargetPosition = new Vector2(44.0f, 100.0f);
            mainMenuStartSpriteNormal.TargetSize = new Vector2(16.0f, 13.0f);
            mainMenuStartSpriteNormal.Visible = true;
            myGame.Components.Add(mainMenuStartSpriteNormal);

            mainMenuStartSpriteHovered = new SpriteGameVisual(game);
            mainMenuStartSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            mainMenuStartSpriteHovered.Enabled = false;
            mainMenuStartSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            mainMenuStartSpriteHovered.SourceImage = myGame.TilesTexture;
            mainMenuStartSpriteHovered.SourceRectangle = new Rectangle(0, 224, 16, 13);
            mainMenuStartSpriteHovered.TargetPosition = new Vector2(44.0f, 100.0f);
            mainMenuStartSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            mainMenuStartSpriteHovered.Visible = true;
            myGame.Components.Add(mainMenuStartSpriteHovered);

            mainMenuHelpSpriteNormal = new SpriteGameVisual(game);
            mainMenuHelpSpriteNormal.DrawOrder = (int)DisplayLayer.UI;
            mainMenuHelpSpriteNormal.Enabled = false;
            mainMenuHelpSpriteNormal.UpdateOrder = (int)UpdateLayer.Last;
            mainMenuHelpSpriteNormal.SourceImage = myGame.TilesTexture;
            mainMenuHelpSpriteNormal.SourceRectangle = new Rectangle(64, 208, 16, 13);
            mainMenuHelpSpriteNormal.TargetPosition = new Vector2(62.0f, 100.0f);
            mainMenuHelpSpriteNormal.TargetSize = new Vector2(16.0f, 13.0f);
            mainMenuHelpSpriteNormal.Visible = true;
            myGame.Components.Add(mainMenuHelpSpriteNormal);

            mainMenuHelpSpriteHovered = new SpriteGameVisual(game);
            mainMenuHelpSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            mainMenuHelpSpriteHovered.Enabled = false;
            mainMenuHelpSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            mainMenuHelpSpriteHovered.SourceImage = myGame.TilesTexture;
            mainMenuHelpSpriteHovered.SourceRectangle = new Rectangle(64, 224, 16, 13);
            mainMenuHelpSpriteHovered.TargetPosition = new Vector2(62.0f, 100.0f);
            mainMenuHelpSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            mainMenuHelpSpriteHovered.Visible = true;
            myGame.Components.Add(mainMenuHelpSpriteHovered);

            mainMenuConfigureSpriteNormal = new SpriteGameVisual(game);
            mainMenuConfigureSpriteNormal.DrawOrder = (int)DisplayLayer.UI;
            mainMenuConfigureSpriteNormal.Enabled = false;
            mainMenuConfigureSpriteNormal.UpdateOrder = (int)UpdateLayer.Last;
            mainMenuConfigureSpriteNormal.SourceImage = myGame.TilesTexture;
            mainMenuConfigureSpriteNormal.SourceRectangle = new Rectangle(48, 208, 16, 13);
            mainMenuConfigureSpriteNormal.TargetPosition = new Vector2(80.0f, 100.0f);
            mainMenuConfigureSpriteNormal.TargetSize = new Vector2(16.0f, 13.0f);
            mainMenuConfigureSpriteNormal.Visible = true;
            myGame.Components.Add(mainMenuConfigureSpriteNormal);

            mainMenuConfigureSpriteHovered = new SpriteGameVisual(game);
            mainMenuConfigureSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            mainMenuConfigureSpriteHovered.Enabled = false;
            mainMenuConfigureSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            mainMenuConfigureSpriteHovered.SourceImage = myGame.TilesTexture;
            mainMenuConfigureSpriteHovered.SourceRectangle = new Rectangle(48, 224, 16, 13);
            mainMenuConfigureSpriteHovered.TargetPosition = new Vector2(80.0f, 100.0f);
            mainMenuConfigureSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            mainMenuConfigureSpriteHovered.Visible = true;
            myGame.Components.Add(mainMenuConfigureSpriteHovered);

            mainMenuQuitSpriteNormal = new SpriteGameVisual(game);
            mainMenuQuitSpriteNormal.DrawOrder = (int)DisplayLayer.UI;
            mainMenuQuitSpriteNormal.Enabled = false;
            mainMenuQuitSpriteNormal.UpdateOrder = (int)UpdateLayer.Last;
            mainMenuQuitSpriteNormal.SourceImage = myGame.TilesTexture;
            mainMenuQuitSpriteNormal.SourceRectangle = new Rectangle(32, 208, 16, 13);
            mainMenuQuitSpriteNormal.TargetPosition = new Vector2(98.0f, 100.0f);
            mainMenuQuitSpriteNormal.TargetSize = new Vector2(16.0f, 13.0f);
            mainMenuQuitSpriteNormal.Visible = true;
            myGame.Components.Add(mainMenuQuitSpriteNormal);

            mainMenuQuitSpriteHovered = new SpriteGameVisual(game);
            mainMenuQuitSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            mainMenuQuitSpriteHovered.Enabled = false;
            mainMenuQuitSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            mainMenuQuitSpriteHovered.SourceImage = myGame.TilesTexture;
            mainMenuQuitSpriteHovered.SourceRectangle = new Rectangle(32, 224, 16, 13);
            mainMenuQuitSpriteHovered.TargetPosition = new Vector2(98.0f, 100.0f);
            mainMenuQuitSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            mainMenuQuitSpriteHovered.Visible = true;
            myGame.Components.Add(mainMenuQuitSpriteHovered);

            // Help
            helpMenuBackSpriteHovered = new SpriteGameVisual(game);
            helpMenuBackSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            helpMenuBackSpriteHovered.Enabled = false;
            helpMenuBackSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            helpMenuBackSpriteHovered.SourceImage = myGame.TilesTexture;
            helpMenuBackSpriteHovered.SourceRectangle = new Rectangle(16, 224, 16, 13);
            helpMenuBackSpriteHovered.TargetPosition = new Vector2(72.0f, 112.0f);
            helpMenuBackSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            helpMenuBackSpriteHovered.Visible = true;
            myGame.Components.Add(helpMenuBackSpriteHovered);
            
            // Game Over
            gameOverMenuBackSpriteHovered = new SpriteGameVisual(game);
            gameOverMenuBackSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            gameOverMenuBackSpriteHovered.Enabled = false;
            gameOverMenuBackSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            gameOverMenuBackSpriteHovered.SourceImage = myGame.TilesTexture;
            gameOverMenuBackSpriteHovered.SourceRectangle = new Rectangle(16, 224, 16, 13);
            gameOverMenuBackSpriteHovered.TargetPosition = new Vector2(72.0f, 112.0f);
            gameOverMenuBackSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            gameOverMenuBackSpriteHovered.Visible = true;
            myGame.Components.Add(gameOverMenuBackSpriteHovered);
            
            // Game Win
            gameWinMenuBackSpriteHovered = new SpriteGameVisual(game);
            gameWinMenuBackSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            gameWinMenuBackSpriteHovered.Enabled = false;
            gameWinMenuBackSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            gameWinMenuBackSpriteHovered.SourceImage = myGame.TilesTexture;
            gameWinMenuBackSpriteHovered.SourceRectangle = new Rectangle(16, 224, 16, 13);
            gameWinMenuBackSpriteHovered.TargetPosition = new Vector2(72.0f, 112.0f);
            gameWinMenuBackSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            gameWinMenuBackSpriteHovered.Visible = true;
            myGame.Components.Add(gameWinMenuBackSpriteHovered);

            // Configure
            configureMenuSfxToggleOffSpriteNormal = new SpriteGameVisual(game);
            configureMenuSfxToggleOffSpriteNormal.DrawOrder = (int)DisplayLayer.UI;
            configureMenuSfxToggleOffSpriteNormal.Enabled = false;
            configureMenuSfxToggleOffSpriteNormal.UpdateOrder = (int)UpdateLayer.Last;
            configureMenuSfxToggleOffSpriteNormal.SourceImage = myGame.TilesTexture;
            configureMenuSfxToggleOffSpriteNormal.SourceRectangle = new Rectangle(80, 208, 16, 13);
            configureMenuSfxToggleOffSpriteNormal.TargetPosition = new Vector2(105.0f, 56.0f);
            configureMenuSfxToggleOffSpriteNormal.TargetSize = new Vector2(16.0f, 13.0f);
            configureMenuSfxToggleOffSpriteNormal.Visible = true;
            myGame.Components.Add(configureMenuSfxToggleOffSpriteNormal);

            configureMenuSfxToggleOffSpriteHovered = new SpriteGameVisual(game);
            configureMenuSfxToggleOffSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            configureMenuSfxToggleOffSpriteHovered.Enabled = false;
            configureMenuSfxToggleOffSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            configureMenuSfxToggleOffSpriteHovered.SourceImage = myGame.TilesTexture;
            configureMenuSfxToggleOffSpriteHovered.SourceRectangle = new Rectangle(80, 224, 16, 13);
            configureMenuSfxToggleOffSpriteHovered.TargetPosition = new Vector2(105.0f, 56.0f);
            configureMenuSfxToggleOffSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            configureMenuSfxToggleOffSpriteHovered.Visible = true;
            myGame.Components.Add(configureMenuSfxToggleOffSpriteHovered);

            configureMenuSfxToggleOnSpriteNormal = new SpriteGameVisual(game);
            configureMenuSfxToggleOnSpriteNormal.DrawOrder = (int)DisplayLayer.UI;
            configureMenuSfxToggleOnSpriteNormal.Enabled = false;
            configureMenuSfxToggleOnSpriteNormal.UpdateOrder = (int)UpdateLayer.Last;
            configureMenuSfxToggleOnSpriteNormal.SourceImage = myGame.TilesTexture;
            configureMenuSfxToggleOnSpriteNormal.SourceRectangle = new Rectangle(96, 208, 16, 13);
            configureMenuSfxToggleOnSpriteNormal.TargetPosition = new Vector2(105.0f, 56.0f);
            configureMenuSfxToggleOnSpriteNormal.TargetSize = new Vector2(16.0f, 13.0f);
            configureMenuSfxToggleOnSpriteNormal.Visible = true;
            myGame.Components.Add(configureMenuSfxToggleOnSpriteNormal);

            configureMenuSfxToggleOnSpriteHovered = new SpriteGameVisual(game);
            configureMenuSfxToggleOnSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            configureMenuSfxToggleOnSpriteHovered.Enabled = false;
            configureMenuSfxToggleOnSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            configureMenuSfxToggleOnSpriteHovered.SourceImage = myGame.TilesTexture;
            configureMenuSfxToggleOnSpriteHovered.SourceRectangle = new Rectangle(96, 224, 16, 13);
            configureMenuSfxToggleOnSpriteHovered.TargetPosition = new Vector2(105.0f, 56.0f);
            configureMenuSfxToggleOnSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            configureMenuSfxToggleOnSpriteHovered.Visible = true;
            myGame.Components.Add(configureMenuSfxToggleOnSpriteHovered);

            configureMenuMusicToggleOffSpriteNormal = new SpriteGameVisual(game);
            configureMenuMusicToggleOffSpriteNormal.DrawOrder = (int)DisplayLayer.UI;
            configureMenuMusicToggleOffSpriteNormal.Enabled = false;
            configureMenuMusicToggleOffSpriteNormal.UpdateOrder = (int)UpdateLayer.Last;
            configureMenuMusicToggleOffSpriteNormal.SourceImage = myGame.TilesTexture;
            configureMenuMusicToggleOffSpriteNormal.SourceRectangle = new Rectangle(80, 208, 16, 13);
            configureMenuMusicToggleOffSpriteNormal.TargetPosition = new Vector2(105.0f, 90.0f);
            configureMenuMusicToggleOffSpriteNormal.TargetSize = new Vector2(16.0f, 13.0f);
            configureMenuMusicToggleOffSpriteNormal.Visible = true;
            myGame.Components.Add(configureMenuMusicToggleOffSpriteNormal);

            configureMenuMusicToggleOffSpriteHovered = new SpriteGameVisual(game);
            configureMenuMusicToggleOffSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            configureMenuMusicToggleOffSpriteHovered.Enabled = false;
            configureMenuMusicToggleOffSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            configureMenuMusicToggleOffSpriteHovered.SourceImage = myGame.TilesTexture;
            configureMenuMusicToggleOffSpriteHovered.SourceRectangle = new Rectangle(80, 224, 16, 13);
            configureMenuMusicToggleOffSpriteHovered.TargetPosition = new Vector2(105.0f, 90.0f);
            configureMenuMusicToggleOffSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            configureMenuMusicToggleOffSpriteHovered.Visible = true;
            myGame.Components.Add(configureMenuMusicToggleOffSpriteHovered);

            configureMenuMusicToggleOnSpriteNormal = new SpriteGameVisual(game);
            configureMenuMusicToggleOnSpriteNormal.DrawOrder = (int)DisplayLayer.UI;
            configureMenuMusicToggleOnSpriteNormal.Enabled = false;
            configureMenuMusicToggleOnSpriteNormal.UpdateOrder = (int)UpdateLayer.Last;
            configureMenuMusicToggleOnSpriteNormal.SourceImage = myGame.TilesTexture;
            configureMenuMusicToggleOnSpriteNormal.SourceRectangle = new Rectangle(96, 208, 16, 13);
            configureMenuMusicToggleOnSpriteNormal.TargetPosition = new Vector2(105.0f, 90.0f);
            configureMenuMusicToggleOnSpriteNormal.TargetSize = new Vector2(16.0f, 13.0f);
            configureMenuMusicToggleOnSpriteNormal.Visible = true;
            myGame.Components.Add(configureMenuMusicToggleOnSpriteNormal);

            configureMenuMusicToggleOnSpriteHovered = new SpriteGameVisual(game);
            configureMenuMusicToggleOnSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            configureMenuMusicToggleOnSpriteHovered.Enabled = false;
            configureMenuMusicToggleOnSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            configureMenuMusicToggleOnSpriteHovered.SourceImage = myGame.TilesTexture;
            configureMenuMusicToggleOnSpriteHovered.SourceRectangle = new Rectangle(96, 224, 16, 13);
            configureMenuMusicToggleOnSpriteHovered.TargetPosition = new Vector2(105.0f, 90.0f);
            configureMenuMusicToggleOnSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            configureMenuMusicToggleOnSpriteHovered.Visible = true;
            myGame.Components.Add(configureMenuMusicToggleOnSpriteHovered);

            configureMenuBackSpriteNormal = new SpriteGameVisual(game);
            configureMenuBackSpriteNormal.DrawOrder = (int)DisplayLayer.UI;
            configureMenuBackSpriteNormal.Enabled = false;
            configureMenuBackSpriteNormal.UpdateOrder = (int)UpdateLayer.Last;
            configureMenuBackSpriteNormal.SourceImage = myGame.TilesTexture;
            configureMenuBackSpriteNormal.SourceRectangle = new Rectangle(16, 208, 16, 13);
            configureMenuBackSpriteNormal.TargetPosition = new Vector2(72.0f, 112.0f);
            configureMenuBackSpriteNormal.TargetSize = new Vector2(16.0f, 13.0f);
            configureMenuBackSpriteNormal.Visible = true;
            myGame.Components.Add(configureMenuBackSpriteNormal);

            configureMenuBackSpriteHovered = new SpriteGameVisual(game);
            configureMenuBackSpriteHovered.DrawOrder = (int)DisplayLayer.UI;
            configureMenuBackSpriteHovered.Enabled = false;
            configureMenuBackSpriteHovered.UpdateOrder = (int)UpdateLayer.Last;
            configureMenuBackSpriteHovered.SourceImage = myGame.TilesTexture;
            configureMenuBackSpriteHovered.SourceRectangle = new Rectangle(16, 224, 16, 13);
            configureMenuBackSpriteHovered.TargetPosition = new Vector2(72.0f, 112.0f);
            configureMenuBackSpriteHovered.TargetSize = new Vector2(16.0f, 13.0f);
            configureMenuBackSpriteHovered.Visible = true;
            myGame.Components.Add(configureMenuBackSpriteHovered);

            #endregion
        }

        public override void Update(GameTime gameTime)
        {
            // Check for key release
            if (lastMenuInputKey == KeysPressedLast.MoveUpPressedLast && !myGame.MoveUpPressed && !lastMenuInputReleased)
            {
                lastMenuInputKey = KeysPressedLast.None;
                lastMenuInputReleased = true;
            }
            else if (lastMenuInputKey == KeysPressedLast.MoveDownPressedLast && !myGame.MoveDownPressed && !lastMenuInputReleased)
            {
                lastMenuInputKey = KeysPressedLast.None;
                lastMenuInputReleased = true;
            }
            else if (lastMenuInputKey == KeysPressedLast.MoveLeftPressedLast && !myGame.MoveLeftPressed && !lastMenuInputReleased)
            {
                lastMenuInputKey = KeysPressedLast.None;
                lastMenuInputReleased = true;
            }
            else if (lastMenuInputKey == KeysPressedLast.MoveRightPressedLast && !myGame.MoveRightPressed && !lastMenuInputReleased)
            {
                lastMenuInputKey = KeysPressedLast.None;
                lastMenuInputReleased = true;
            }
            else if (lastMenuInputKey == KeysPressedLast.FirePressedLast && !myGame.FirePressed && !lastMenuInputReleased)
            {
                lastMenuInputKey = KeysPressedLast.None;
                lastMenuInputReleased = true;
            }
            else if (lastMenuInputKey == KeysPressedLast.PausePressedLast && !myGame.PausePressed && !lastMenuInputReleased)
            {
                lastMenuInputKey = KeysPressedLast.None;
                lastMenuInputReleased = true;
            }
            else if (lastMenuInputKey == KeysPressedLast.BackPressedLast && !myGame.BackPressed && !lastMenuInputReleased)
            {
                lastMenuInputKey = KeysPressedLast.None;
                lastMenuInputReleased = true;
            }

            if (myGame.CurrentGameMode == GameMode.Title)
            {
                if (myGame.FirePressed & lastMenuInputReleased)
                {
                    lastMenuInputKey = KeysPressedLast.FirePressedLast;
                    lastMenuInputReleased = false;

                    if (myGame.SfxOn)
                    {
                        myGame.MenuSelectSound.Play();
                    }

                    switch (mainMenuHoveredIndex)
                    {
                        case 0:
                            myGame.StartNewGame();
                            break;
                        case 1:
                            myGame.SwitchToHelpScreen();
                            break;
                        case 2:
                            myGame.SwitchToConfigureScreen();
                            break;
                        case 3:
                            myGame.QuitApplication();
                            break;
                        default:
                            break;
                    }
                }

                if (myGame.MoveRightPressed & lastMenuInputReleased)
                {
                    lastMenuInputKey = KeysPressedLast.MoveRightPressedLast;
                    lastMenuInputReleased = false;

                    if (myGame.SfxOn)
                    {
                        myGame.MenuNavSound.Play();
                    }

                    mainMenuHoveredIndex += 1;
                    if (mainMenuHoveredIndex > 3)
                    {
                        mainMenuHoveredIndex = 0;
                    }

                    UpdateMainMenuDisplay();
                }
                else if (myGame.MoveLeftPressed & lastMenuInputReleased)
                {
                    lastMenuInputKey = KeysPressedLast.MoveLeftPressedLast;
                    lastMenuInputReleased = false;

                    if (myGame.SfxOn)
                    {
                        myGame.MenuNavSound.Play();
                    }

                    mainMenuHoveredIndex -= 1;
                    if (mainMenuHoveredIndex < 0)
                    {
                        mainMenuHoveredIndex = 3;
                    }

                    UpdateMainMenuDisplay();
                }

                ShowSpritesForTitleMode(true);
            }
            else
            {
                ShowSpritesForTitleMode(false);
            }

            if (myGame.CurrentGameMode == GameMode.Help)
            {
                if (myGame.FirePressed & lastMenuInputReleased)
                {
                    lastMenuInputKey = KeysPressedLast.FirePressedLast;
                    lastMenuInputReleased = false;

                    if (myGame.SfxOn)
                    {
                        myGame.MenuSelectSound.Play();
                    }

                    myGame.SwitchToTitleScreen();
                }

                ShowBackButtonSpriteForHelpMode(true);
            }
            else
            {
                ShowBackButtonSpriteForHelpMode(false);
            }

            if (myGame.CurrentGameMode == GameMode.GameOver)
            {
                if (myGame.FirePressed & lastMenuInputReleased)
                {
                    lastMenuInputKey = KeysPressedLast.FirePressedLast;
                    lastMenuInputReleased = false;

                    if (myGame.SfxOn)
                    {
                        myGame.MenuSelectSound.Play();
                    }

                    myGame.SwitchToTitleScreen();
                }

                ShowBackButtonSpriteForGameOverMode(true);
            }
            else
            {
                ShowBackButtonSpriteForGameOverMode(false);
            }

            if (myGame.CurrentGameMode == GameMode.GameWin)
            {
                if (myGame.FirePressed & lastMenuInputReleased)
                {
                    lastMenuInputKey = KeysPressedLast.FirePressedLast;
                    lastMenuInputReleased = false;

                    if (myGame.SfxOn)
                    {
                        myGame.MenuSelectSound.Play();
                    }

                    myGame.SwitchToTitleScreen();
                }

                ShowBackButtonSpriteForGameWinMode(true);
            }
            else
            {
                ShowBackButtonSpriteForGameWinMode(false);
            }

            if (myGame.CurrentGameMode == GameMode.Configure)
            {
                if (myGame.FirePressed & lastMenuInputReleased)
                {
                    lastMenuInputKey = KeysPressedLast.FirePressedLast;
                    lastMenuInputReleased = false;

                    if (myGame.SfxOn)
                    {
                        myGame.MenuSelectSound.Play();
                    }

                    if (configureMenuHoveredIndex == 0)
                    {
                        myGame.SfxOn = !myGame.SfxOn;
                    }
                    else if (configureMenuHoveredIndex == 1)
                    {
                        myGame.MusicOn = !myGame.MusicOn;
                    }
                    else if (configureMenuHoveredIndex == 2)
                    {
                        myGame.SwitchToTitleScreen();
                    }
                }

                if (myGame.MoveUpPressed & lastMenuInputReleased)
                {
                    lastMenuInputKey = KeysPressedLast.MoveUpPressedLast;
                    lastMenuInputReleased = false;

                    if (myGame.SfxOn)
                    {
                        myGame.MenuNavSound.Play();
                    }

                    configureMenuHoveredIndex -= 1;
                    if (configureMenuHoveredIndex < 0)
                    {
                        configureMenuHoveredIndex = 2;
                    }

                    UpdateConfigureMenuDisplay();
                }
                else if (myGame.MoveDownPressed & lastMenuInputReleased)
                {
                    lastMenuInputKey = KeysPressedLast.MoveDownPressedLast;
                    lastMenuInputReleased = false;

                    if (myGame.SfxOn)
                    {
                        myGame.MenuNavSound.Play();
                    }

                    configureMenuHoveredIndex += 1;
                    if (configureMenuHoveredIndex > 2)
                    {
                        configureMenuHoveredIndex = 0;
                    }

                    UpdateConfigureMenuDisplay();
                }
                else if (myGame.MoveLeftPressed & lastMenuInputReleased)
                {
                    lastMenuInputKey = KeysPressedLast.MoveLeftPressedLast;
                    lastMenuInputReleased = false;

                    if (myGame.SfxOn)
                    {
                        myGame.MenuNavSound.Play();
                    }

                    configureMenuHoveredIndex -= 1;
                    if (configureMenuHoveredIndex < 0)
                    {
                        configureMenuHoveredIndex = 2;
                    }

                    UpdateConfigureMenuDisplay();
                }
                else if (myGame.MoveRightPressed & lastMenuInputReleased)
                {
                    lastMenuInputKey = KeysPressedLast.MoveRightPressedLast;
                    lastMenuInputReleased = false;

                    if (myGame.SfxOn)
                    {
                        myGame.MenuNavSound.Play();
                    }

                    configureMenuHoveredIndex += 1;
                    if (configureMenuHoveredIndex > 2)
                    {
                        configureMenuHoveredIndex = 0;
                    }

                    UpdateConfigureMenuDisplay();
                }

                ShowSpritesForConfigurationMode(true);
            }
            else
            {
                ShowSpritesForConfigurationMode(false);
            }

            base.Update(gameTime);
        }

        void ShowSpritesForTitleMode(bool flag)
        {
            if (mainMenuStartSpriteNormal.Visible != flag)
            {
                mainMenuStartSpriteNormal.Visible = flag;
            }

            if (mainMenuStartSpriteHovered.Visible != flag)
            {
                mainMenuStartSpriteHovered.Visible = flag;
            }

            if (mainMenuHelpSpriteNormal.Visible != flag)
            {
                mainMenuHelpSpriteNormal.Visible = flag;
            }

            if (mainMenuHelpSpriteHovered.Visible != flag)
            {
                mainMenuHelpSpriteHovered.Visible = flag;
            }

            if (mainMenuConfigureSpriteNormal.Visible != flag)
            {
                mainMenuConfigureSpriteNormal.Visible = flag;
            }

            if (mainMenuConfigureSpriteHovered.Visible != flag)
            {
                mainMenuConfigureSpriteHovered.Visible = flag;
            }

            if (mainMenuQuitSpriteNormal.Visible != flag)
            {
                mainMenuQuitSpriteNormal.Visible = flag;
            }

            if (mainMenuQuitSpriteHovered.Visible != flag)
            {
                mainMenuQuitSpriteHovered.Visible = flag;
            }

            if (flag)
            {
                UpdateMainMenuDisplay();
            }
        }

        void ShowBackButtonSpriteForHelpMode(bool flag)
        {
            if (helpMenuBackSpriteHovered.Visible != flag)
            {
                helpMenuBackSpriteHovered.Visible = flag;
            }
        }

        void ShowBackButtonSpriteForGameOverMode(bool flag)
        {
            if (gameOverMenuBackSpriteHovered.Visible != flag)
            {
                gameOverMenuBackSpriteHovered.Visible = flag;
            }
        }

        void ShowBackButtonSpriteForGameWinMode(bool flag)
        {
            if (gameWinMenuBackSpriteHovered.Visible != flag)
            {
                gameWinMenuBackSpriteHovered.Visible = flag;
            }
        }

        void ShowSpritesForConfigurationMode(bool flag)
        {
            if (configureMenuSfxToggleOffSpriteNormal.Visible != flag)
            {
                configureMenuSfxToggleOffSpriteNormal.Visible = flag;
            }

            if (configureMenuSfxToggleOffSpriteHovered.Visible != flag)
            {
                configureMenuSfxToggleOffSpriteHovered.Visible = flag;
            }

            if (configureMenuMusicToggleOffSpriteNormal.Visible != flag)
            {
                configureMenuMusicToggleOffSpriteNormal.Visible = flag;
            }

            if (configureMenuMusicToggleOffSpriteNormal.Visible != flag)
            {
                configureMenuMusicToggleOffSpriteNormal.Visible = flag;
            }

            if (configureMenuSfxToggleOnSpriteNormal.Visible != flag)
            {
                configureMenuSfxToggleOnSpriteNormal.Visible = flag;
            }

            if (configureMenuSfxToggleOnSpriteHovered.Visible != flag)
            {
                configureMenuSfxToggleOnSpriteHovered.Visible = flag;
            }

            if (configureMenuMusicToggleOffSpriteNormal.Visible != flag)
            {
                configureMenuMusicToggleOffSpriteNormal.Visible = flag;
            }

            if (configureMenuMusicToggleOffSpriteHovered.Visible != flag)
            {
                configureMenuMusicToggleOffSpriteHovered.Visible = flag;
            }

            if (configureMenuMusicToggleOnSpriteNormal.Visible != flag)
            {
                configureMenuMusicToggleOnSpriteNormal.Visible = flag;
            }

            if (configureMenuMusicToggleOnSpriteHovered.Visible != flag)
            {
                configureMenuMusicToggleOnSpriteHovered.Visible = flag;
            }

            if (configureMenuBackSpriteNormal.Visible != flag)
            {
                configureMenuBackSpriteNormal.Visible = flag;
            }

            if (configureMenuBackSpriteHovered.Visible != flag)
            {
                configureMenuBackSpriteHovered.Visible = flag;
            }

            if (flag)
            {
                UpdateConfigureMenuDisplay();
            }
        }

        void UpdateMainMenuDisplay()
        {
            mainMenuStartSpriteNormal.Visible = mainMenuHoveredIndex == 0 ? false : true;
            mainMenuStartSpriteHovered.Visible = mainMenuHoveredIndex == 0 ? true : false;

            mainMenuHelpSpriteNormal.Visible = mainMenuHoveredIndex == 1 ? false : true;
            mainMenuHelpSpriteHovered.Visible = mainMenuHoveredIndex == 1 ? true : false;

            mainMenuConfigureSpriteNormal.Visible = mainMenuHoveredIndex == 2 ? false : true;
            mainMenuConfigureSpriteHovered.Visible = mainMenuHoveredIndex == 2 ? true : false;

            mainMenuQuitSpriteNormal.Visible = mainMenuHoveredIndex == 3 ? false : true;
            mainMenuQuitSpriteHovered.Visible = mainMenuHoveredIndex == 3 ? true : false;
        }

        void UpdateConfigureMenuDisplay()
        {
            if(myGame.SfxOn)
            {
                configureMenuSfxToggleOnSpriteNormal.Visible = configureMenuHoveredIndex == 0 ? false : true;
                configureMenuSfxToggleOnSpriteHovered.Visible = configureMenuHoveredIndex == 0 ? true : false;

                configureMenuSfxToggleOffSpriteNormal.Visible = false;
                configureMenuSfxToggleOffSpriteHovered.Visible = false;
            }
            else
            {
                configureMenuSfxToggleOffSpriteNormal.Visible = configureMenuHoveredIndex == 0 ? false : true;
                configureMenuSfxToggleOffSpriteHovered.Visible = configureMenuHoveredIndex == 0 ? true : false;

                configureMenuSfxToggleOnSpriteNormal.Visible = false;
                configureMenuSfxToggleOnSpriteHovered.Visible = false;
            }

            if (myGame.MusicOn)
            { 
                configureMenuMusicToggleOnSpriteNormal.Visible = configureMenuHoveredIndex == 1 ? false : true;
                configureMenuMusicToggleOnSpriteHovered.Visible = configureMenuHoveredIndex == 1 ? true : false;

                configureMenuMusicToggleOffSpriteNormal.Visible = false;
                configureMenuMusicToggleOffSpriteHovered.Visible = false;
            }
            else
            {
                configureMenuMusicToggleOffSpriteNormal.Visible = configureMenuHoveredIndex == 1 ? false : true;
                configureMenuMusicToggleOffSpriteHovered.Visible = configureMenuHoveredIndex == 1 ? true : false;

                configureMenuMusicToggleOnSpriteNormal.Visible = false;
                configureMenuMusicToggleOnSpriteHovered.Visible = false;
            }

            configureMenuBackSpriteNormal.Visible = configureMenuHoveredIndex == 2 ? false : true;
            configureMenuBackSpriteHovered.Visible = configureMenuHoveredIndex == 2 ? true : false;
        }
    }
}
